#include <stdarg.h>
#include <string>
#include <string.h>
#include <stdlib.h>
#include <Core/CFByteArray.h>
#include <Core/CFLog.h>

CFByteArray::CFByteArray()
    : m_data(NULL)
    , m_strLen(0) {
}

CFByteArray::CFByteArray(const CFChar* str, int size)
    : m_data(NULL)
    , m_strLen(0) {
    create((const CFUChar*)str, size);
}

CFByteArray::CFByteArray(const CFUChar* str, int size)
    : m_data(NULL)
    , m_strLen(0) {
    create(str, size);
}

CFByteArray::CFByteArray(const CFByteArray& other)
    : m_data(NULL) {
    create(other.m_data, other.m_strLen);
}

CFByteArray::~CFByteArray() {
	if (NULL != m_data) {
        delete [] m_data;
        m_data = NULL;
	}
	m_strLen = 0;
}

CFUInt CFByteArray::size() {
    return m_strLen;
}

CFUChar* CFByteArray::data() {
    return m_data;
}

const CFUChar* CFByteArray::data() const {
    return m_data;
}

CFList<CFByteArray> CFByteArray::split(CFUChar sep) {
    CFList<CFByteArray> ret;
    int startPos = 0;

    while (startPos < m_strLen) {
        int endPos = indexOf(sep, startPos);
        if (0 > endPos) {
            ret.append(sub(startPos, -1));
            break;
        }
        if (startPos < endPos) {
            ret.append(sub(startPos, endPos - startPos));
        }
        startPos = endPos + 1;
    }
    return ret;
}

CFByteArray CFByteArray::sub(CFUInt pos, CFInt len) {
    if (NULL == m_data) {
        return "";
    }
	if (0 == len) {
        return CFByteArray();
    }
    CFUInt index = pos;
    if (0 > len) {
        len = m_strLen - pos;
    }

    CFInt dataLen = len + 1;
    CFUChar* strret = new CFUChar[dataLen];
    memset(strret, 0x0, sizeof(CFUChar) * (dataLen));
    CFUChar chr;
    int i = 0;
    CFInt wordCount = 0;
    while (i < dataLen && i < m_strLen && wordCount < len) {
        chr = *(m_data + i + index);
        strret[i] = chr;
        i++;
        wordCount++;
    }

    CFByteArray stringRet = CFByteArray(strret);
    delete [] strret;
    return stringRet;
}

CFInt CFByteArray::indexOf(CFUChar chr, int from, bool caseSensitive) {
    CFUChar strtmp[] = {chr};
    return indexOf(strtmp, from, caseSensitive);
}

CFInt CFByteArray::indexOf(const CFByteArray& str, int from, bool caseSensitive) {
    if (str.isEmpty() || isEmpty() || from >= m_strLen) {
        return -1;
    }

    int i = 0;
    int firstIndex = -1;
    CFUChar srcChr = *m_data;
    CFUChar destChr = *str.m_data;
    bool isFound = false;
    for (i = from; i < m_strLen; ++i) {
        srcChr = *(m_data + i);
        destChr = *(str.m_data);
        firstIndex = -1;
        isFound = false;
        if (charEqual(destChr, srcChr, caseSensitive)) {
            firstIndex = i;
            isFound = true;
            for (int j = 0; j < str.m_strLen; ++j) {
                if ((firstIndex + j) >= m_strLen) {
                    isFound = false;
                    break;
                }
                srcChr = *(m_data + i + j);
                destChr = *(str.m_data + j);
                if (!charEqual(destChr, srcChr, caseSensitive)) {
                    isFound = false;
                    break;
                }
            }
            if (isFound) {
                break;
            }
        }
    }
    if (0 <= firstIndex && m_strLen > firstIndex && isFound) {
        return firstIndex;
    }
    return -1;
}

bool CFByteArray::isEmpty() const {
    return (m_strLen == 0 || NULL == m_data || (*(m_data) == '\0'));
}

bool CFByteArray::equal(const CFByteArray& other) {
    return (0 == compare(other));
}

CFInt CFByteArray::compare(const CFByteArray& other) {
    if (NULL == m_data || NULL == other.m_data) {
        return -1;
    }
    int compLength = 0;
    if ((m_strLen != other.m_strLen)) {
        compLength = (m_strLen < other.m_strLen) ? m_strLen : other.m_strLen;
    } else {
        compLength = m_strLen;
    }

    int ret = memcmp(m_data, other.m_data, compLength);
    if (0 == ret) {
        if (m_strLen != other.m_strLen) {
            return (m_strLen - other.m_strLen);
        }
    }
    return ret;
}

void CFByteArray::display() {
	CFLog::d("CFByteArray", "LINE: %d, FUNC: %s -----> data(%s), strlen(%d)", __LINE__, __PRETTY_FUNCTION__, m_data, m_strLen);
}

void CFByteArray::create(const CFUChar* strdata, int size) {
    if (NULL == strdata) {
        return;
    }
    if (NULL != m_data) {
        delete [] m_data;
        m_data = NULL;
    }

    if (0 >= size) {
        m_strLen = dataLength(strdata);
    } else {
        m_strLen = size + 1;
    }
    m_data = new CFUChar[m_strLen + 1];
    memset(m_data, 0x0, sizeof(CFUChar) * (m_strLen + 1));
    memcpy(m_data, strdata, sizeof(CFUChar) * (m_strLen));
}

CFUInt CFByteArray::dataLength(const CFUChar* strdata) {
    if (NULL == strdata) {
        return 0;
    }
    int cnt = 0;
    CFUChar chr = *(strdata + cnt);
    while (0 != chr) {
        cnt++;
        chr = *(strdata + cnt);
    }
    return cnt;
}

bool CFByteArray::charEqual(CFUChar destChr, CFUChar srcChr, bool caseSensitive) {
    int charOffSet = caseSensitive ? 0 : 32;
    if ('a' <= destChr && 'z' >= destChr) {
        if (srcChr == destChr || srcChr == destChr - charOffSet) {
            return true;
        }
    } else if ('A' <= destChr && 'Z' >= destChr) {
        if (srcChr == destChr || srcChr == destChr + charOffSet) {
            return true;
        }
    } else {
        if (destChr == srcChr) {
            return true;
        }
    }
    return false;
}

CFByteArray operator+(const CFByteArray& str1, const CFByteArray& str2) {
    if (NULL == str1.m_data || NULL == str2.m_data) {
        return CFByteArray();
    }
    CFInt fullLength = str1.m_strLen + str2.m_strLen + 1;
    CFUChar* stringRet = new CFUChar[fullLength];
    CFInt i = 0;
    memset(stringRet, 0x0, sizeof(CFUChar) * fullLength);

    memccpy(stringRet, str1.m_data, 0, sizeof(CFUChar) * str1.m_strLen);
    memccpy(stringRet + sizeof(CFUChar) * str1.m_strLen, str2.m_data, 0, sizeof(CFUChar) * str2.m_strLen);

    CFByteArray ret = CFByteArray(stringRet);
    delete [] stringRet;
    return ret;
}

CFByteArray& CFByteArray::operator=(const CFByteArray& str1) {
    if (NULL == str1.m_data) {
        return *this;
    }
    create(str1.m_data);
    return *this;
}

CFByteArray& CFByteArray::operator+=(const CFByteArray& str1) {
    if (NULL == str1.m_data) {
        return *this;
    }
    CFByteArray tmp = *(this) + str1;
    create(tmp.m_data);
    return *this;
}

CFByteArray operator+(const CFByteArray& str1, const CFUChar* str2) {
    return CFByteArray(str1 + CFByteArray(str2));
}

CFByteArray operator+(const CFUChar* str1, const CFByteArray& str2) {
    return CFByteArray(CFByteArray(str1) + str2);
}

CFByteArray operator+(const CFByteArray& str1, const CFChar* str2) {
    return CFByteArray(str1 + CFByteArray(str2));
}

CFByteArray operator+(const CFChar* str1, const CFByteArray& str2) {
    return CFByteArray(CFByteArray(str1) + str2);
}

bool CFByteArray::operator==(const CFByteArray& other) {
    return equal(other);
}

bool CFByteArray::operator!=(const CFByteArray& other) {
    return (!equal(other));
}

bool CFByteArray::operator>(const CFByteArray& other) {
    return (0 < compare(other));
}

bool CFByteArray::operator>=(const CFByteArray& other) {
    return (0 <= compare(other));
}

bool CFByteArray::operator<(const CFByteArray& other) {
    return (0 > compare(other));
}

bool CFByteArray::operator<=(const CFByteArray& other) {
    return (0 >= compare(other));
}
