#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string>

#include <Core/CFLog.h>

#define LOG_BUFFER_SIZE (128)
#define LOG_TIME_SIZE (16)

static std::string s_log_string[] = {
    "INFO",
    "DEBUG",
    "WARN",
    "ERROR",
    "FATAL"
};

CFLog* CFLog::s_inst = NULL;

void CFLog::addLevelMask(unsigned int mask) {
    CFLog* loginst = getInst();
    if (NULL != loginst) {
        loginst->addMask(mask & CFLOG_MASK_GATE);
    }
}

void CFLog::removeLevelMask(unsigned int mask) {
    CFLog* loginst = getInst();
    if (NULL != loginst) {
        loginst->removeMask(mask & CFLOG_MASK_GATE);
    }
}

void CFLog::i(const char* tag, const char* format, ...) {
    CFLog* loginst = getInst();
    if (NULL != format && NULL != loginst) {
        va_list ap;
        va_start(ap, format);
        loginst->log(CFLOGLV_INFO, tag, format, ap);
        va_end(ap);
    }
}

void CFLog::d(const char* tag, const char* format, ...) {
    CFLog* loginst = getInst();
    if (NULL != format && NULL != loginst) {
        va_list ap;
        va_start(ap, format);
        loginst->log(CFLOGLV_DEBUG, tag, format, ap);
        va_end(ap);
    }
}

void CFLog::w(const char* tag, const char* format, ...) {
    CFLog* loginst = getInst();
    if (NULL != format && NULL != loginst) {
        va_list ap;
        va_start(ap, format);
        loginst->log(CFLOGLV_WARNING, tag, format, ap);
        va_end(ap);
    }
}

void CFLog::e(const char* tag, const char* format, ...) {
    CFLog* loginst = getInst();
    if (NULL != format && NULL != loginst) {
        va_list ap;
        va_start(ap, format);
        loginst->log(CFLOGLV_ERROR, tag, format, ap);
        va_end(ap);
    }
}

void CFLog::f(const char* tag, const char* format, ...) {
    CFLog* loginst = getInst();
    if (NULL != format && NULL != loginst) {
        va_list ap;
        va_start(ap, format);
        loginst->log(CFLOGLV_FATAL, tag, format, ap);
        va_end(ap);
    }
}

void CFLog::logCommon(CFLogLevel level, const char* tag, const char* format, va_list ap) {
    CFLog* loginst = getInst();
    if (NULL != format && NULL != loginst) {
        loginst->log(level, tag, format, ap);
    }
}

CFLog::CFLog()
    : m_mask() {

}

CFLog::CFLog(const CFLog& ) {

}

CFLog* CFLog::getInst() {
    if (NULL == s_inst) {
        s_inst = new CFLog();
    }
    return s_inst;
}

void CFLog::log(CFLogLevel level, const char* tag,  const char* format, va_list ap) {
    switch (level) {
    case CFLOGLV_INFO:
        if (0 == (m_mask & CFLOGMSK_INFO)) {
            return;
        }
        break;
    case CFLOGLV_DEBUG:
        if (0 == (m_mask & CFLOGMSK_DEBUG)) {
            return;
        }
        break;
    default:
        break;
    }

    char buffer[LOG_BUFFER_SIZE];

    memset(buffer, 0x0, LOG_BUFFER_SIZE);

    vsnprintf(buffer, LOG_BUFFER_SIZE - 1, format, ap);

    char* timestr = currentTimeStr();
    printf("%s [%s] %s: %s\n", timestr, ::s_log_string[level].c_str(), tag, buffer);
    free(timestr);
}

void CFLog::addMask(unsigned int mask) {
    m_mask = m_mask | mask | CFLOGMSK_WARNING | CFLOGMSK_ERROR | CFLOGMSK_FATAL;
}

void CFLog::removeMask(unsigned int mask) {
    m_mask = (m_mask & (~mask)) | CFLOGMSK_WARNING | CFLOGMSK_ERROR | CFLOGMSK_FATAL;
}

char* CFLog::currentTimeStr() {
    struct timespec ts;
    struct tm* localtm;
    clock_gettime(CLOCK_REALTIME, &ts);
    localtm = localtime(&ts.tv_sec);
    char* timestr = (char*)calloc(sizeof(char), LOG_TIME_SIZE);
    snprintf(timestr, LOG_TIME_SIZE, "[%02d:%02d:%02d.%03ld]", localtm->tm_hour, localtm->tm_min, localtm->tm_sec, ts.tv_nsec/1000000);
    return timestr;
}

CF_EXPORT void cf_enable_debug() {
    CFLog::addLevelMask(CFLOGMSK_DEBUG);
}

CF_EXPORT void cf_disable_debug() {
    CFLog::removeLevelMask(CFLOGMSK_DEBUG);
}

CF_EXPORT void cf_enable_info() {
    CFLog::addLevelMask(CFLOGMSK_INFO);
}

CF_EXPORT void cf_disable_info() {
    CFLog::removeLevelMask(CFLOGMSK_INFO);
}

CF_EXPORT void cf_log_Common(const char* logtag, CFLogLevel level, const char* format, ...) {
    va_list ap;
    va_start(ap, format);
    CFLog::logCommon(level, logtag, format, ap);
    va_end(ap);
}