// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// A class representing an adjustable bounded value.
#include "Gui/CFAdjustment.h"
#include "gtkextends/CFGtkAdjustment.h"

CF_NAMESPACE_BEGIN

CFAdjustment::CFAdjustment(double value,
                           double lower,
                           double upper,
                           double step_increment,
                           double page_increment,
                           double page_size) {
    setRealObject(new CFGtkAdjustment(
        value, lower, upper, step_increment, page_increment, page_size));
}

CFAdjustment::~CFAdjustment() {
    setRealObject(NULL);
}

void CFAdjustment::changed() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->changed();
    }
}

void CFAdjustment::valueChanged() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->value_changed();
    }
}

void CFAdjustment::clampPage(double lower, double upper) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->clamp_page(lower, upper);
    }
}

void CFAdjustment::setValue(double value) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->set_value(value);
    }
}

double CFAdjustment::getValue() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double value = -1;
    if (NULL != obj) {
        value = obj->get_value();
    }
    return value;
}

void CFAdjustment::setLower(double lower) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->set_lower(lower);
    }
}

double CFAdjustment::getLower() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double lower = -1;
    if (NULL != obj) {
        lower = obj->get_lower();
    }
    return lower;
}

void CFAdjustment::setUpper(double upper) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->set_upper(upper);
    }
}

double CFAdjustment::getUpper() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double upper = -1;
    if (NULL != obj) {
        upper = obj->get_upper();
    }
    return upper;
}

void CFAdjustment::setStepIncrement(double step_increment) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->set_step_increment(step_increment);
    }
}

double CFAdjustment::getStepIncrement() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double step_increment = -1;
    if (NULL != obj) {
        step_increment = obj->get_step_increment();
    }
    return step_increment;
}

void CFAdjustment::setPageIncrement(double page_increment) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->set_page_increment(page_increment);
    }
}

double CFAdjustment::getPageIncrement() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double page_increment = -1;
    if (NULL != obj) {
        page_increment = obj->get_page_increment();
    }
    return page_increment;
}

void CFAdjustment::setPageSize(double page_size) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->set_page_size(page_size);
    }
}

double CFAdjustment::getPageSize() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double page_size = -1;
    if (NULL != obj) {
        page_size = obj->get_page_size();
    }
    return page_size;
}

void CFAdjustment::configure(double value,
                             double lower,
                             double upper,
                             double step_increment,
                             double page_increment,
                             double page_size) {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->configure(
            value, lower, upper, step_increment, page_increment, page_size);
    }
}

double CFAdjustment::getMinimumIncrement() {
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    double minimum_increment = -1;
    if (NULL != obj) {
        minimum_increment = obj->get_minimum_increment();
    }
    return minimum_increment;
}

CFAdjustment* CFAdjustment::create(double value,
                                   double lower,
                                   double upper,
                                   double step_increment,
                                   double page_increment,
                                   double page_size) {
    return new CFAdjustment(
        value, lower, upper, step_increment, page_increment, page_size);
}

void CFAdjustment::onRealObjectInit() {
    CFObject::onRealObjectInit();
    Gtk::Adjustment* obj = static_cast<Gtk::Adjustment*>(getObj());
    if (NULL != obj) {
        obj->signal_changed().connect(
            sigc::mem_fun(*this, &CFAdjustment::onChanged));
        obj->signal_value_changed().connect(
            sigc::mem_fun(*this, &CFAdjustment::onValueChanged));
    }
}

void CFAdjustment::onRealObjectFinalize() {
}

sigc::signal<void> CFAdjustment::signalChanged() {
    return m_changedSignal;
}

sigc::signal<void> CFAdjustment::signalValueChanged() {
    return m_changedValueSignal;
}

void CFAdjustment::onChanged() {
    signalChanged().emit();
}

void CFAdjustment::onValueChanged() {
    signalValueChanged().emit();
}

CF_NAMESPACE_END
