#include <gtkmm.h>

#include "Gui/CFButton.h"
#include "PrivCommon/CFConvertor.h"

CF_NAMESPACE_BEGIN

CFButton::CFButton()
    : CFBin() {
    setRealObject(new Gtk::Button());
}

CFButton::CFButton(const Glib::ustring& label, bool mnemonic)
    : CFBin() {
    setRealObject(new Gtk::Button(label, mnemonic));
}

CFButton::~CFButton() {
    setRealObject(NULL);
}

void CFButton::setRelief(CFReliefStyle newstyle) {
    Gtk::Button* obj = static_cast<Gtk::Button*>(getObj());
    if (NULL != obj) {
        obj->set_relief(toGtkReliefStyle(newstyle));
    }
}

CFReliefStyle CFButton::getRelief() {
    Gtk::Button* obj = static_cast<Gtk::Button*>(getObj());
    if (NULL != obj) {
        return toCdosReliefStyle(obj->get_relief());
    }
    return CFRELIEF_NORMAL;
}

void CFButton::setLabel(const Glib::ustring& label) {
    Gtk::Button* obj = static_cast<Gtk::Button*>(getObj());
    if (NULL != obj) {
        obj->set_label(label);
    }
}

Glib::ustring CFButton::getLabel() {
    Gtk::Button* obj = static_cast<Gtk::Button*>(getObj());
    if (NULL != obj) {
        return obj->get_label();
    }
    return "";
}

sigc::signal<void> CFButton::signalPressed() {
    return m_pressedSignal;
}

sigc::signal<void> CFButton::signalReleased() {
    return m_releasedSignal;
}

sigc::signal<void> CFButton::signalClicked() {
    return m_clickedSignal;
}

sigc::signal<void> CFButton::signalEnter() {
    return m_enterSignal;
}

sigc::signal<void> CFButton::signalLeave() {
    return m_leaveSignal;
}

sigc::signal<void> CFButton::signalActivate() {
    return m_activateSignal;
}

void CFButton::onRealObjectInit() {
    CFBin::onRealObjectInit();
    Gtk::Button* obj = static_cast<Gtk::Button*>(getObj());
    if (NULL != obj) {
        obj->signal_pressed().connect(
            sigc::mem_fun(*this, &CFButton::onPressed));
        obj->signal_released().connect(
            sigc::mem_fun(*this, &CFButton::onReleased));
        obj->signal_clicked().connect(
            sigc::mem_fun(*this, &CFButton::onClicked));
        obj->signal_enter().connect(sigc::mem_fun(*this, &CFButton::onEnter));
        obj->signal_leave().connect(sigc::mem_fun(*this, &CFButton::onLeave));
        obj->signal_activate().connect(
            sigc::mem_fun(*this, &CFButton::onActivate));
    }
}

void CFButton::onRealObjectFinalize() {
    CFBin::onRealObjectFinalize();
}

void CFButton::onPressed() {
    signalPressed().emit();
}

void CFButton::onReleased() {
    signalReleased().emit();
}

void CFButton::onClicked() {
    signalClicked().emit();
}

void CFButton::onEnter() {
    signalEnter().emit();
}

void CFButton::onLeave() {
    signalLeave().emit();
}

void CFButton::onActivate() {
    signalActivate().emit();
}

CF_NAMESPACE_END
