// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// A widget used to choose from a list of items.
#include "Gui/CFComboBox.h"
#include <gtkmm.h>
#include <PrivCommon/CFConvertor.h>

CF_NAMESPACE_BEGIN

CFComboBox::CFComboBox(bool has_entry) {
    setRealObject(new Gtk::ComboBox(has_entry));
}

CFComboBox::~CFComboBox() {
    setRealObject(NULL);
}

void CFComboBox::setWrapWidth(int width) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_wrap_width(width);
    }
    return;
}

int CFComboBox::getWrapWidth() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        int width = obj->get_wrap_width();
        return width;
    }
    return -1;
}

void CFComboBox::setRowSpanColumn(int row_span) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_row_span_column(row_span);
    }
    return;
}

int CFComboBox::getRowSpanColumn() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        int column = obj->get_row_span_column();
        return column;
    }
    return -1;
}

void CFComboBox::setColumnSpanColumn(int column_span) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_column_span_column(column_span);
    }
    return;
}

int CFComboBox::getColumnSpanColumn() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        int column = obj->get_column_span_column();
        return column;
    }
    return -1;
}

void CFComboBox::setAddTearoffs(bool add_tearoffs) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_add_tearoffs(add_tearoffs);
    }
    return;
}

bool CFComboBox::getAddTearoffs() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        bool res = obj->get_add_tearoffs();
        return res;
    }
    return true;
}

Glib::ustring CFComboBox::getTitle() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        Glib::ustring str = obj->get_title();
        return str;
    }
    return "";
}

void CFComboBox::setTitle(const Glib::ustring& title) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_title(title);
    }
    return;
}

bool CFComboBox::getFocusOnClick() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        bool res = obj->get_focus_on_click();
        return res;
    }
    return true;
}

void CFComboBox::setFocusOnClick(bool focus_on_click) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_focus_on_click(focus_on_click);
    }
    return;
}

int CFComboBox::getActiveRowNumber() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        int number = obj->get_active_row_number();
        return number;
    }
    return -1;
}

void CFComboBox::setActive(int index) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_active(index);
    }
    return;
}

void CFComboBox::unsetActive() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->unset_active();
    }
    return;
}

void CFComboBox::unsetRowSeparatorFunc() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->unset_row_separator_func();
    }
    return;
}

void CFComboBox::setButtonSensitivity(CFSensitivityType sensitivity) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        Gtk::SensitivityType g_sensitivity = toGtkSensitivityType(sensitivity);
        obj->set_button_sensitivity(g_sensitivity);
    }
    return;
}

CFSensitivityType CFComboBox::getButtonSensitivity() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        Gtk::SensitivityType g_sensitivity = obj->get_button_sensitivity();
        CFSensitivityType sensitivity = toCdosSensitivityType(g_sensitivity);
        return sensitivity;
    }
    return CFSENSITIVITY_AUTO;
}

bool CFComboBox::getHasEntry() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        bool res = obj->get_has_entry();
        return res;
    }
    return false;
}

void CFComboBox::setEntryTextColumn(int text_column) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_entry_text_column(text_column);
    }
    return;
}

int CFComboBox::getEntryTextColumn() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        int column = obj->get_entry_text_column();
        return column;
    }
    return -1;
}

void CFComboBox::setPopupFixedWidth(bool fixed) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_popup_fixed_width(fixed);
    }
    return;
}

bool CFComboBox::getPopupFixedWidth() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        bool res = obj->get_popup_fixed_width();
        return res;
    }
    return true;
}

void CFComboBox::popup() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->popup();
    }
    return;
}

void CFComboBox::popup(const Glib::RefPtr<Gdk::Device>& device) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->popup(device);
    }
    return;
}

void CFComboBox::popdown() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->popdown();
    }
    return;
}

int CFComboBox::getIdColumn() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        int column = obj->get_id_column();
        return column;
    }
    return -1;
}

void CFComboBox::setIdColumn(int id_column) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->set_id_column(id_column);
    }
    return;
}

Glib::ustring CFComboBox::getActiveId() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        Glib::ustring id_str = obj->get_active_id();
        return id_str;
    }
    return "";
}

void CFComboBox::getActiveId(const Glib::ustring& active_id) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        obj->get_active_id(active_id);
    }
    return;
}

bool CFComboBox::setActiveId(const Glib::ustring& active_id) {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        bool is_fond = obj->set_active_id(active_id);
        return is_fond;
    }
    return false;
}

CFEntry* CFComboBox::getEntry() {
    Gtk::ComboBox* obj = static_cast<Gtk::ComboBox*>(getObj());
    if (NULL != obj) {
        Gtk::Entry* entry = obj->get_entry();
        CFEntry* cf_entry = static_cast<CFEntry*>(getCFObject(entry));
        return cf_entry;
    }
    return NULL;
}


CF_NAMESPACE_END
