#include "Gui/CFContainer.h"
#include <gtkmm.h>
#include "PrivCommon/PrivDefines.h"
#include "gtkextends/CFGtkContainer.h"

CF_NAMESPACE_BEGIN

class CFContainerPriv {
public:
    explicit CFContainerPriv(CFContainer* container)
        : m_container(container) {
    }

    void onForeach(Gtk::Widget& widget);

    CFContainer* m_container;
};

CFContainer::CFContainer()
    : CFWidget()
    , m_priv(new CFContainerPriv(this)) {
    setRealObject(new CFGtkContainer());
}

CFContainer::~CFContainer() {
    setRealObject(NULL);
}

void CFContainer::add(CFWidget& widget) {
    Gtk::Container* obj = static_cast<Gtk::Container*>(getObj());
    Gtk::Widget* wobj = static_cast<Gtk::Widget*>(widget.getObj());
    if (NULL != obj && NULL != wobj) {
        obj->add(*wobj);
    }
}

void CFContainer::remove(CFWidget& widget) {
    Gtk::Container* obj = static_cast<Gtk::Container*>(getObj());
    Gtk::Widget* wobj = static_cast<Gtk::Widget*>(widget.getObj());
    if (NULL != obj && NULL != wobj) {
        obj->remove(*wobj);
    }
}

void CFContainer::foreach (ForeachSlot slot) {
    Gtk::Container* obj = static_cast<Gtk::Container*>(getObj());
    if (NULL != obj) {
        signalForeach().connect(slot);
        obj->foreach (sigc::mem_fun(*m_priv, &CFContainerPriv::onForeach));
        slot.disconnect();
    }
}

void CFContainer::forall(ForeachSlot slot) {
    Gtk::Container* obj = static_cast<Gtk::Container*>(getObj());
    if (NULL != obj) {
        signalForeach().connect(slot);
        obj->forall(sigc::mem_fun(*m_priv, &CFContainerPriv::onForeach));
        slot.disconnect();
    }
}

bool CFContainer::isContainer() {
    return true;
}

void CFContainer::addCssStylePath(const Glib::ustring& path, CFUInt priority) {
    m_cssStylePath = path;
    m_cssStylePriority = priority;
    CFWidget::addCssStylePath(path, priority);
    foreach(sigc::mem_fun(*this, &CFContainer::onApplyCssData))
        ;
}

void CFContainer::addCssStyleData(const Glib::ustring& data, CFUInt priority) {
    m_cssStyleData = data;
    m_cssStylePriority = priority;
    CFWidget::addCssStyleData(data, priority);
    foreach(sigc::mem_fun(*this, &CFContainer::onApplyCssData))
        ;
}

void CFContainer::onApplyCssPath(CFWidget* widget) {
    if (NULL != widget) {
        widget->addCssStylePath(m_cssStylePath, m_cssStylePriority);
    }
}

void CFContainer::onApplyCssData(CFWidget* widget) {
    if (NULL != widget) {
        widget->addCssStyleData(m_cssStyleData, m_cssStylePriority);
    }
}

sigc::signal<void, CFWidget*> CFContainer::signalForeach() {
    return m_foreachSignal;
}

void CFContainerPriv::onForeach(Gtk::Widget& widget) {
    CFWidget* wobj = static_cast<CFWidget*>(m_container->getCFObject(&widget));
    if (NULL != wobj) {
        m_container->signalForeach().emit(wobj);
    }
}

CF_NAMESPACE_END
