#include "Gui/CFEntry.h"
#include <gtkmm.h>

CF_NAMESPACE_BEGIN

CFEntry::CFEntry()
    : CFWidget() {
    setRealObject(new Gtk::Entry());
}

CFEntry::~CFEntry() {
    setRealObject(NULL);
}

void CFEntry::setVisibility(bool visible) {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        obj->set_visibility(visible);
    }
}

bool CFEntry::getVisibility() {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        return obj->get_visibility();
    }
    return true;
}

void CFEntry::setInvisibleChar(char ch) {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        obj->set_invisible_char(ch);
    }
}

void CFEntry::unsetInvisibleChar() {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        obj->unset_invisible_char();
    }
}

char CFEntry::getInvisibleChar() {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        return obj->get_invisible_char();
    }
    return '\0';
}

void CFEntry::setPlaceholderText(const Glib::ustring& text) {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        obj->set_placeholder_text(text);
    }
}

void CFEntry::setText(const Glib::ustring& text) {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        obj->set_text(text);
    }
}

Glib::ustring CFEntry::getText() {
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        return obj->get_text();
    }
    return "";
}

sigc::signal< void > CFEntry::signalActivate() {
    return m_activateSignal;
}

void CFEntry::onRealObjectInit() {
    CFWidget::onRealObjectInit();
    Gtk::Entry* obj = static_cast<Gtk::Entry*>(getObj());
    if (NULL != obj) {
        obj->signal_activate().connect(sigc::mem_fun(*this, &CFEntry::onActivate));
    }
}

void CFEntry::onActivate() {
    signalActivate().emit();
}

CF_NAMESPACE_END
