#include "Gui/CFLabel.h"
#include <gtkmm.h>
#include "CFConvertor.h"

CF_NAMESPACE_BEGIN

CFLabel::CFLabel()
    : CFMisc() {
    setRealObject(new Gtk::Label());
}

CFLabel::CFLabel(const Glib::ustring& label, bool mnemonic)
    : CFMisc() {
    setRealObject(new Gtk::Label(label, mnemonic));
}

CFLabel::CFLabel(const Glib::ustring& label,
                 float xalign,
                 float yalign,
                 bool mnemonic)
    : CFMisc() {
    setRealObject(new Gtk::Label(label, xalign, yalign, mnemonic));
}

CFLabel::CFLabel(const Glib::ustring& label,
                 CFAlign xalign,
                 CFAlign yalign,
                 bool mnemonic)
    : CFMisc() {
    setRealObject(new Gtk::Label(
        label, toGtkAlign(xalign), toGtkAlign(yalign), mnemonic));
}

CFLabel::~CFLabel() {
    setRealObject(NULL);
}

void CFLabel::setText(const Glib::ustring& str) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_text(str);
    }
}

Glib::ustring CFLabel::getText() {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        return obj->get_text();
    }
    return "";
}

void CFLabel::setLabel(const Glib::ustring& str) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_label(str);
    }
}

Glib::ustring CFLabel::getLabel() {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        return obj->get_label();
    }
    return "";
}

void CFLabel::setMarkup(const Glib::ustring& str) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_markup(str);
    }
}

void CFLabel::setUseMarkup(bool setting) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_use_markup(setting);
    }
}

void CFLabel::setEllipsize(Pango::EllipsizeMode mode) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_ellipsize(mode);
    }
}

void CFLabel::setJustify(CFJustification type) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        Gtk::Justification jtype = toGtkJustification(type);
        obj->set_justify(jtype);
    }
}

CFJustification CFLabel::getJustify() {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        Gtk::Justification jtype_ = obj->get_justify();
        CFJustification jtype = toCdosJustification(jtype_);
        return jtype;
    }
    return CFJUSTIFY_LEFT;  // default
}

void CFLabel::setLineWrap(bool wrap) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_line_wrap(wrap);
    }
    return;
}

void CFLabel::setLineWrapMode(Pango::WrapMode wrap_mode) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_line_wrap_mode(wrap_mode);
    }
    return;
}

void CFLabel::setLines(int lines) {
    Gtk::Label* obj = static_cast<Gtk::Label*>(getObj());
    if (NULL != obj) {
        obj->set_lines(lines);
    }
    return;
}

CF_NAMESPACE_END
