// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// As item add in CFListWidget.
#include "Gui/CFListWidget.h"
#include <gtkmm.h>
#include "PrivCommon/CFConvertor.h"

CF_NAMESPACE_BEGIN
// CFListWidgetPriv
class CFListWidgetPriv {
public:
    explicit CFListWidgetPriv(CFListWidget* list_widget)
        : m_listWidget(list_widget) {
    }

    // signal handler
    void onRowSelected(Gtk::ListBoxRow* box);

    void onRowActivated(Gtk::ListBoxRow* box);

    // slot handler
    bool onFilter(Gtk::ListBoxRow* row);

    int onSort(Gtk::ListBoxRow* row1, Gtk::ListBoxRow* row2);

    void onUpdateHeader(Gtk::ListBoxRow* row1, Gtk::ListBoxRow* row2);

    void onForeach(Gtk::ListBoxRow* row);

private:
    CFListWidget* m_listWidget;
};

// CFListWidgetPriv Signal
void CFListWidgetPriv::onRowSelected(Gtk::ListBoxRow* box) {
    CFListWidgetItem* list_widget
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(box));
    m_listWidget->signalItemSelected().emit(list_widget);
}

void CFListWidgetPriv::onRowActivated(Gtk::ListBoxRow* box) {
    CFListWidgetItem* list_widget
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(box));
    m_listWidget->signalItemActivated().emit(list_widget);
}

// CFListWidgetPriv Slot
bool CFListWidgetPriv::onFilter(Gtk::ListBoxRow* row) {
    CFListWidgetItem* item
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(row));
    bool res = m_listWidget->signalFilter().emit(item);
    return res;
}

int CFListWidgetPriv::onSort(Gtk::ListBoxRow* row1, Gtk::ListBoxRow* row2) {
    CFListWidgetItem* item1
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(row1));
    CFListWidgetItem* item2
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(row2));
    int res = m_listWidget->signalSort().emit(item1, item2);
    return res;
}

void CFListWidgetPriv::onUpdateHeader(Gtk::ListBoxRow* row1,
                                        Gtk::ListBoxRow* row2) {
    CFListWidgetItem* item1
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(row1));
    CFListWidgetItem* item2
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(row2));
    m_listWidget->signalUpdateHeader().emit(item1, item2);
}

void CFListWidgetPriv::onForeach(Gtk::ListBoxRow* row) {
    CFListWidgetItem* item
        = static_cast<CFListWidgetItem*>(CFObject::getCFObject(row));
    m_listWidget->signalForeach().emit(item);
}

// CFListWidget
CFListWidget::CFListWidget() {
    setRealObject(new Gtk::ListBox());
}

CFListWidget::~CFListWidget() {
    setRealObject(NULL);
}

void CFListWidget::prepend(CFWidget& child) {
    Gtk::Widget* wchild = static_cast<Gtk::Widget*>(child.getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->prepend(*wchild);
    }
    return;
}

void CFListWidget::insert(CFWidget& child, int position) {
    Gtk::Widget* wchild = static_cast<Gtk::Widget*>(child.getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->insert(*wchild, position);
    }
    return;
}

void CFListWidget::append(CFWidget& child) {
    Gtk::Widget* wchild = static_cast<Gtk::Widget*>(child.getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->append(*wchild);
    }
    return;
}

CFListWidgetItem* CFListWidget::getSelectedItem() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        Gtk::ListBoxRow* row = obj->get_selected_row();
        CFListWidgetItem* item
            = static_cast<CFListWidgetItem*>(getCFObject(row));
        return item;
    }
    return NULL;
}

CFListWidgetItem* CFListWidget::getItemAtIndex(int index) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        Gtk::ListBoxRow* row = obj->get_row_at_index(index);
        CFListWidgetItem* item
            = static_cast<CFListWidgetItem*>(getCFObject(row));
        return item;
    }
    return NULL;
}

CFListWidgetItem* CFListWidget::getItemAtY(int y) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        Gtk::ListBoxRow* row = obj->get_row_at_y(y);
        CFListWidgetItem* item
            = static_cast<CFListWidgetItem*>(getCFObject(row));
        return item;
    }
    return NULL;
}

void CFListWidget::selectItem(CFListWidgetItem& item) {
    Gtk::ListBoxRow* row = static_cast<Gtk::ListBoxRow*>(item.getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->select_row(*row);
    }
    return;
}

void CFListWidget::unselectItem() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unselect_row();
    }
    return;
}

void CFListWidget::unselectItem(CFListWidgetItem* item) {
    Gtk::ListBoxRow* row = static_cast<Gtk::ListBoxRow*>(item->getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unselect_row(row);
    }
    return;
}

void CFListWidget::setPlaceholder(CFWidget& placeholder) {
    Gtk::Widget* wplaceholder = static_cast<Gtk::Widget*>(placeholder.getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->set_placeholder(*wplaceholder);
    }
    return;
}

void CFListWidget::unsetPlaceholder() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unset_placeholder();
    }
    return;
}

void CFListWidget::selectedForeach(SlotForeach& slot) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        signalForeach().connect(slot);
        obj->selected_foreach(
            sigc::mem_fun(*m_priv, &CFListWidgetPriv::onForeach));
        slot.disconnect();
    }
}

std::vector<CFListWidgetItem*> CFListWidget::getSelectItems() {
    std::vector<CFListWidgetItem*> cflist = {};
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        std::vector<Gtk::ListBoxRow*> list = obj->get_selected_rows();
        for (auto row : list) {
            CFListWidgetItem* item
                = static_cast<CFListWidgetItem*>(getCFObject(row));
            cflist.push_back(item);
        }
    }
    return cflist;
}

void CFListWidget::selectAll() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->select_all();
    }
    return;
}

void CFListWidget::unselectAll() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unselect_all();
    }
    return;
}

void CFListWidget::setSelectionMode(CFSelectionMode mode) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        Gtk::SelectionMode gmode = toGtkSelectionMode(mode);
        obj->set_selection_mode(gmode);
    }
    return;
}

CFSelectionMode CFListWidget::getSelectionMode() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        Gtk::SelectionMode gmode = obj->get_selection_mode();
        CFSelectionMode mode = toCdosSelectionMode(gmode);
        return mode;
    }
    return CFSELECTION_NONE;
}

void CFListWidget::setFilterFunc(SlotFilter& slot) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        signalFilter().connect(slot);
        obj->set_filter_func(
            sigc::mem_fun(*m_priv, &CFListWidgetPriv::onFilter));
        slot.disconnect();
    }
    return;
}

void CFListWidget::unsetFilterFunc() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unset_filter_func();
    }
    return;
}

void CFListWidget::setSortFunc(SlotSort& slot) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        signalSort().connect(slot);
        obj->set_sort_func(sigc::mem_fun(*m_priv, &CFListWidgetPriv::onSort));
        slot.disconnect();
    }
    return;
}

void CFListWidget::unsetSortFunc() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unset_sort_func();
    }
    return;
}

void CFListWidget::setHeaderFunc(SlotUpdateHeader& slot) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        signalUpdateHeader().connect(slot);
        obj->set_header_func(
            sigc::mem_fun(*m_priv, &CFListWidgetPriv::onUpdateHeader));
        slot.disconnect();
    }
    return;
}

void CFListWidget::unsetHeaderFunc() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->unset_header_func();
    }
    return;
}

void CFListWidget::invalidateFilter() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->invalidate_filter();
    }
    return;
}

void CFListWidget::invalidateSort() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->invalidate_sort();
    }
    return;
}

void CFListWidget::setActivateOnSingleClick(bool single) {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->set_activate_on_single_click(single);
    }
    return;
}

bool CFListWidget::getActiveOnSingleClick() {
    bool res = false;
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        res = obj->get_activate_on_single_click();
    }
    return res;
}

void CFListWidget::dragUnhighlightItem() {
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->drag_unhighlight_row();
    }
    return;
}

void CFListWidget::dragHighlightItem(CFListWidgetItem& item) {
    Gtk::ListBoxRow* row = static_cast<Gtk::ListBoxRow*>(item.getObj());
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->drag_highlight_row(*row);
    }
    return;
}

// void CFListWidget::bind_model(Glib::RefPtr<Gio::ListModel>& model,
//                 SlotCreateWidget<Glib::Object>& slot_create_widget) {
//     return;
// }
// 
// void CFListWidget::bind_list_store(const
// Glib::RefPtr<Gio::ListStore<T_item>>& store,
//                      T_slot&& slot_create_widget) {
//     return;
// }

// CFListWidget Signal
sigc::signal<void, CFListWidgetItem*> CFListWidget::signalItemSelected() {
    return m_itemSelectedSignal;
}

sigc::signal<void, CFListWidgetItem*> CFListWidget::signalItemActivated() {
    return m_itemActivatedSignal;
}

sigc::signal<void> CFListWidget::signalSelectedItemsChanged() {
    return m_selectedItemsChangedSignal;
}

void CFListWidget::onSelectedItemsChanged() {
    signalSelectedItemsChanged().emit();
}

sigc::signal<void, CFListWidgetItem*> CFListWidget::signalForeach() {
    return m_foreachSignal;
}

sigc::signal<bool, CFListWidgetItem*> CFListWidget::signalFilter() {
    return m_filterSignal;
}

sigc::signal<int, CFListWidgetItem*, CFListWidgetItem*>
CFListWidget::signalSort() {
    return m_sortSignal;
}

sigc::signal<void, CFListWidgetItem*, CFListWidgetItem*>
CFListWidget::signalUpdateHeader() {
    return m_updateHeaderSignal;
}

// init and finalize
void CFListWidget::onRealObjectInit() {
    CFContainer::onRealObjectInit();
    Gtk::ListBox* obj = static_cast<Gtk::ListBox*>(getObj());
    if (NULL != obj) {
        obj->signal_row_selected().connect(
            sigc::mem_fun(*m_priv, &CFListWidgetPriv::onRowSelected));
        obj->signal_row_activated().connect(
            sigc::mem_fun(*m_priv, &CFListWidgetPriv::onRowActivated));
        obj->signal_selected_rows_changed().connect(
            sigc::mem_fun(*this, &CFListWidget::onSelectedItemsChanged));
    }
}

void CFListWidget::onRealObjectFinalize() {
}

CF_NAMESPACE_END
