#include "Gui/CFMenu.h"
#include "Gui/CFMenuItem.h"
#include <gtkmm.h>

CF_NAMESPACE_BEGIN

CFMenu::CFMenu()
    : CFMenuShell() {
    setRealObject(new Gtk::Menu());
}

CFMenu::CFMenu(const Glib::RefPtr<Gio::MenuModel>& model)
    : CFMenuShell() {
    setRealObject(new Gtk::Menu(model));
}

void CFMenu::popup(CFMenuShell& parent_menu_shell,
                   CFMenuItem& parent_menu_item,
                   const SlotPositionCalc& slot,
                   CFUInt button,
                   CFUInt32 activate_time,
                   const Glib::RefPtr<Gdk::Device>& device) {
    Gtk::MenuShell* parentShell
        = static_cast<Gtk::MenuShell*>(parent_menu_shell.getObj());

    Gtk::MenuItem* parentItem
        = static_cast<Gtk::MenuItem*>(parent_menu_item.getObj());

    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj && NULL != parentItem && NULL != parentShell) {
        obj->popup(
            *parentShell, *parentItem, slot, button, activate_time, device);
    }
}

void CFMenu::popup(const SlotPositionCalc& position_calc_slot,
                   CFUInt button,
                   CFUInt32 activate_time,
                   const Glib::RefPtr<Gdk::Device>& device) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->popup(position_calc_slot, button, activate_time, device);
    }
}

void CFMenu::popup(CFUInt button,
                   CFUInt32 activate_time,
                   const Glib::RefPtr<Gdk::Device>& device) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->popup(button, activate_time, device);
    }
}

void CFMenu::popupAtWidget(CFWidget* widget,
                           Gdk::Gravity widget_anchor,
                           Gdk::Gravity menu_anchor,
                           const GdkEvent* trigger_event) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    Gtk::Widget* wobj = NULL;
    if (NULL != widget) {
        wobj = static_cast<Gtk::Widget*>(widget->getObj());
    }
    if (NULL != obj) {
        obj->popup_at_widget(wobj, widget_anchor, menu_anchor, trigger_event);
    }
}

void CFMenu::popupAtPointer(const GdkEvent* trigger_event) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->popup_at_pointer(trigger_event);
    }
}

void CFMenu::reposition() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->reposition();
    }
}

void CFMenu::popdown() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->popdown();
    }
}

CFMenuItem* CFMenu::getActive() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    CFMenuItem* res = NULL;
    if (NULL != obj) {
        res = static_cast<CFMenuItem*>(
            CFObject::getCFObject(obj->get_active()));
    }
    return res;
}

void CFMenu::setActive(CFUInt index) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_active(index);
    }
}

void CFMenu::setAccelPath(const Glib::ustring& accel_path) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_accel_path(accel_path);
    }
}

Glib::ustring CFMenu::getAccelPath() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        return obj->get_accel_path();
    }
    return "";
}

void CFMenu::detach() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->detach();
    }
}

CFWidget* CFMenu::getAttachWidget() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    CFWidget* res = NULL;
    if (NULL != obj) {
        res = static_cast<CFWidget*>(
            CFObject::getCFObject(obj->get_attach_widget()));
    }
    return res;
}

void CFMenu::setTearoffState(bool torn_off) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_tearoff_state(torn_off);
    }
}

bool CFMenu::getTearoffState() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        return obj->get_tearoff_state();
    }
    return false;
}

void CFMenu::setTitle(const Glib::ustring& title) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_title(title);
    }
}

void CFMenu::unsetTitle() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->unset_title();
    }
}

Glib::ustring CFMenu::getTitle() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        return obj->get_title();
    }
    return "";
}

void CFMenu::setScreen(const Glib::RefPtr<Gdk::Screen>& screen) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_screen(screen);
    }
}

void CFMenu::attach(CFWidget& child,
                    CFUInt left_attach,
                    CFUInt right_attach,
                    CFUInt top_attach,
                    CFUInt bottom_attach) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    Gtk::Widget* wobj = static_cast<Gtk::Widget*>(child.getObj());
    if (NULL != obj && NULL != wobj) {
        obj->attach(
            *wobj, left_attach, right_attach, top_attach, bottom_attach);
    }
}

void CFMenu::setMonitor(int monitor_num) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_monitor(monitor_num);
    }
}

int CFMenu::getMonitor() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        return obj->get_monitor();
    }
    return 0;
}

void CFMenu::reorderChild(CFMenuItem& child, int position) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    Gtk::MenuItem* itemObj = static_cast<Gtk::MenuItem*>(child.getObj());
    if (NULL != obj && NULL != itemObj) {
        obj->reorder_child(*itemObj, position);
    }
}

void CFMenu::setReserveToggleSize(bool reserve_toggle_size) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        obj->set_reserve_toggle_size(reserve_toggle_size);
    }
}

bool CFMenu::getReserveToggleSize() {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    if (NULL != obj) {
        return obj->get_reserve_toggle_size();
    }
    return false;
}

void CFMenu::attachToWidget(CFWidget& attach_widget) {
    Gtk::Menu* obj = static_cast<Gtk::Menu*>(getObj());
    Gtk::Widget* wobj = static_cast<Gtk::Widget*>(attach_widget.getObj());
    if (NULL != obj && NULL != wobj) {
        obj->attach_to_widget(*wobj);
    }
}

CF_NAMESPACE_END
