#include "Gui/CFMenuItem.h"
#include "Gui/CFMenu.h"
#include <gtkmm.h>

CF_NAMESPACE_BEGIN

CFMenuItem::CFMenuItem()
    : CFBin() {
    setRealObject(new Gtk::MenuItem());
}

CFMenuItem::CFMenuItem(CFWidget& widget)
    : CFBin() {
    Gtk::Widget* wobj = static_cast<Gtk::Widget*>(widget.getObj());
    if (NULL != wobj) {
        setRealObject(new Gtk::MenuItem(*wobj));
    } else {
        setRealObject(new Gtk::MenuItem());
    }
}

CFMenuItem::CFMenuItem(const Glib::ustring& label, bool mnemonic)
    : CFBin() {
    setRealObject(new Gtk::MenuItem(label, mnemonic));
}

CFMenuItem::~CFMenuItem() {
    setRealObject(NULL);
}

void CFMenuItem::setSubmenu(CFMenu& submenu) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    Gtk::Menu* menuObj = static_cast<Gtk::Menu*>(submenu.getObj());
    if (NULL != obj && NULL != menuObj) {
        obj->set_submenu(*menuObj);
    }
}

void CFMenuItem::unsetSubmenu() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->unset_submenu();
    }
}

CFMenu* CFMenuItem::getSubmenu() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    CFMenu* ret = NULL;
    if (NULL != obj) {
        ret = static_cast<CFMenu*>(getCFObject(obj->get_submenu()));
    }
    return ret;
}

bool CFMenuItem::hasSubmenu() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        return obj->has_submenu();
    }
    return false;
}

void CFMenuItem::select() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->select();
    }
}

void CFMenuItem::deselect() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->deselect();
    }
}

void CFMenuItem::activate() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->activate();
    }
}

void CFMenuItem::toggleSizeRequest(int& requisition) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->toggle_size_request(requisition);
    }
}

void CFMenuItem::toggleSizeAllocate(int allocation) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->toggle_size_allocate(allocation);
    }
}

void CFMenuItem::setRightJustified(bool right_justified) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->set_right_justified(right_justified);
    }
}

bool CFMenuItem::getRightJustified() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        return obj->get_right_justified();
    }
    return false;
}

void CFMenuItem::setAccelPath(const Glib::ustring& accel_path) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->set_accel_path(accel_path);
    }
}

void CFMenuItem::unsetAccelPath() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->unset_accel_path();
    }
}

Glib::ustring CFMenuItem::getAccelPath() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        return obj->get_accel_path();
    }
    return "";
}

void CFMenuItem::setLabel(const Glib::ustring& label) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->set_label(label);
    }
}

Glib::ustring CFMenuItem::getLabel() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        return obj->get_label();
    }
    return "";
}

void CFMenuItem::setUseUnderline(bool setting) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->set_use_underline(setting);
    }
}

bool CFMenuItem::getUseUnderline() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        return obj->get_use_underline();
    }
    return false;
}

void CFMenuItem::setReserveIndicator(bool reserve) {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->set_reserve_indicator(reserve);
    }
}

bool CFMenuItem::getReserveIndicator() {
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        return obj->get_reserve_indicator();
    }
    return false;
}

sigc::signal<void> CFMenuItem::signalActivate() {
    return m_activateSignal;
}

sigc::signal<void> CFMenuItem::signalActivateItem() {
    return m_activateItemSignal;
}

sigc::signal<void, int*> CFMenuItem::signalToggleSizeRequest() {
    return m_toggleSizeRequestSignal;
}

sigc::signal<void, int> CFMenuItem::signalToggleSizeAllocate() {
    return m_toggleSizeAllocSignal;
}

sigc::signal<void> CFMenuItem::signalSelect() {
    return m_selectSignal;
}

sigc::signal<void> CFMenuItem::signalDeselect() {
    return m_deselectSignal;
}

void CFMenuItem::onRealObjectInit() {
    CFBin::onRealObjectInit();
    Gtk::MenuItem* obj = static_cast<Gtk::MenuItem*>(getObj());
    if (NULL != obj) {
        obj->signal_activate().connect(
            sigc::mem_fun(*this, &CFMenuItem::onActivate));
        obj->signal_activate_item().connect(
            sigc::mem_fun(*this, &CFMenuItem::onActivateItem));
        obj->signal_select().connect(
            sigc::mem_fun(*this, &CFMenuItem::onSelect));
        obj->signal_deselect().connect(
            sigc::mem_fun(*this, &CFMenuItem::onDeselect));
        obj->signal_toggle_size_allocate().connect(
            sigc::mem_fun(*this, &CFMenuItem::onToggleSizeAllocate));
        obj->signal_toggle_size_request().connect(
            sigc::mem_fun(*this, &CFMenuItem::onToggleSizeRequest));
    }
}

void CFMenuItem::onRealObjectFinalize() {
    CFBin::onRealObjectFinalize();
}

void CFMenuItem::onActivate() {
    signalActivate().emit();
}

void CFMenuItem::onActivateItem() {
    signalActivateItem().emit();
}

void CFMenuItem::onToggleSizeRequest(int* requisition) {
    signalToggleSizeRequest().emit(requisition);
}

void CFMenuItem::onToggleSizeAllocate(int allocation) {
    signalToggleSizeAllocate().emit(allocation);
}

void CFMenuItem::onSelect() {
    signalSelect().emit();
}

void CFMenuItem::onDeselect() {
    signalDeselect();
}

CF_NAMESPACE_END
