#include "Gui/CFMenuShell.h"
#include "Gui/CFMenuItem.h"
#include "Gui/CFWindow.h"
#include "gtkextends/CFGtkMenuShell.h"
#include <gtkmm.h>

CF_NAMESPACE_BEGIN

class CFMenuShellPrivate {
public:
    explicit CFMenuShellPrivate(CFMenuShell* meushell)
        : m_menuShell(meushell) {
    }

    void onInsert(Gtk::Widget* widget, int pos);

    CFMenuShell* m_menuShell;
};

CFMenuShell::CFMenuShell()
    : CFContainer()
    , m_priv(new CFMenuShellPrivate(this)) {
    setRealObject(new CFGtkMenuShell());
}

CFMenuShell::~CFMenuShell() {
    setRealObject(NULL);
}

void CFMenuShell::append(CFMenuItem& child) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::MenuItem* itemObj = static_cast<Gtk::MenuItem*>(child.getObj());
    if (NULL != obj && NULL != itemObj) {
        obj->append(*itemObj);
    }
}

void CFMenuShell::prepend(CFMenuItem& child) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::MenuItem* itemObj = static_cast<Gtk::MenuItem*>(child.getObj());
    if (NULL != obj && NULL != itemObj) {
        obj->prepend(*itemObj);
    }
}

void CFMenuShell::insert(CFMenuItem& child, int position) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::MenuItem* itemObj = static_cast<Gtk::MenuItem*>(child.getObj());
    if (NULL != obj && NULL != itemObj) {
        obj->insert(*itemObj, position);
    }
}

void CFMenuShell::selectItem(CFMenuItem& menu_item) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::MenuItem* itemObj = static_cast<Gtk::MenuItem*>(menu_item.getObj());
    if (NULL != obj && NULL != itemObj) {
        obj->select_item(*itemObj);
    }
}

void CFMenuShell::deselect() {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->deselect();
    }
}

void CFMenuShell::activateItem(CFMenuItem& menu_item, bool force_deactivate) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::MenuItem* itemObj = static_cast<Gtk::MenuItem*>(menu_item.getObj());
    if (NULL != obj && NULL != itemObj) {
        obj->activate_item(*itemObj, force_deactivate);
    }
}

void CFMenuShell::selectFirst(bool search_sensitive) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->select_first(search_sensitive);
    }
}

void CFMenuShell::deactivate() {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->deactivate();
    }
}

void CFMenuShell::cancel() {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->cancel();
    }
}

void CFMenuShell::accelerate(CFWindow& window) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::Window* winObj = static_cast<Gtk::Window*>(window.getObj());
    if (NULL != obj && NULL != winObj) {
        obj->accelerate(*winObj);
    }
}

void CFMenuShell::accelerate(CFWidget& parent) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    Gtk::Widget* pobj = static_cast<Gtk::Widget*>(parent.getObj());
    if (NULL != obj && NULL != pobj) {
        obj->accelerate(*pobj);
    }
}

bool CFMenuShell::getTakeFocus() {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        return obj->get_take_focus();
    }
    return false;
}

void CFMenuShell::setTakeFocus(bool take_focus) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->set_take_focus(take_focus);
    }
}

CFWidget* CFMenuShell::getSelectedItem() {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        Gtk::Widget* wobj = obj->get_selected_item();
        if (NULL != wobj) {
            return static_cast<CFWidget*>(getCFObject(wobj));
        }
    }
    return NULL;
}

CFWidget* CFMenuShell::getParentShell() {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        Gtk::Widget* wobj = obj->get_parent_shell();
        if (NULL != wobj) {
            return static_cast<CFWidget*>(getCFObject(wobj));
        }
    }
    return NULL;
}

void CFMenuShell::bindModel(const Glib::RefPtr<Gio::MenuModel>& model,
                            const Glib::ustring& action_namespace,
                            bool with_separators) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->bind_model(model, action_namespace, with_separators);
    }
}

void CFMenuShell::bindModel(const Glib::RefPtr<Gio::MenuModel>& model,
                            bool with_separators) {
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->bind_model(model, with_separators);
    }
}

sigc::signal<void> CFMenuShell::signalDeactivate() {
    return m_deactivateSignal;
}

sigc::signal<void> CFMenuShell::signalSelectionDone() {
    return m_selectionDoneSignal;
}

sigc::signal<bool, int> CFMenuShell::signalMoveSelected() {
    return m_moveSelectedSignal;
}

sigc::signal<void, CFWidget*, int> CFMenuShell::signalInsert() {
    return m_insertSignal;
}

void CFMenuShell::onRealObjectInit() {
    CFContainer::onRealObjectInit();
    Gtk::MenuShell* obj = static_cast<Gtk::MenuShell*>(getObj());
    if (NULL != obj) {
        obj->signal_deactivate().connect(
            sigc::mem_fun(*this, &CFMenuShell::onDeactivate));
        obj->signal_insert().connect(
            sigc::mem_fun(*m_priv, &CFMenuShellPrivate::onInsert));
        obj->signal_move_selected().connect(
            sigc::mem_fun(*this, &CFMenuShell::onMoveSelected));
        obj->signal_selection_done().connect(
            sigc::mem_fun(*this, &CFMenuShell::onSelectionDone));
    }
}

void CFMenuShell::onRealObjectFinalize() {
    CFContainer::onRealObjectFinalize();
}

void CFMenuShell::onDeactivate() {
    signalDeactivate().emit();
}

void CFMenuShell::onSelectionDone() {
    signalSelectionDone().emit();
}

bool CFMenuShell::onMoveSelected(int distance) {
    bool res = signalMoveSelected().emit(distance);
    return res;
}

void CFMenuShellPrivate::onInsert(Gtk::Widget* widget, int pos) {
    if (NULL != m_menuShell) {
        CFWidget* wobj = static_cast<CFWidget*>(CFObject::getCFObject(widget));
        if (NULL != wobj) {
            m_menuShell->signalInsert().emit(wobj, pos);
        }
    }
}

CF_NAMESPACE_END
