// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFNotification is a notification item class.
#include <Gui/CFNotification.h>
#include <Gui/CFEnums.h>
#include <iostream>

CF_NAMESPACE_BEGIN

CFNotification::CFNotification(const Glib::ustring& icon_name,
                               const Glib::ustring& title,
                               const Glib::ustring& content)
    : m_contentBox(CFORIENTATION_VERTICAL, 5)
    , m_actor(CFORIENTATION_HORIZONTAL, 5)
    , m_isNeedShowTime(true) {
    setRelief(CFRELIEF_NONE);  // 去掉浮雕效果
    m_reason = kClicked;
    m_urgency = kUrgencyNormal;
    m_createTime = Glib::DateTime::create_now_local();
    signalClicked().connect(sigc::mem_fun(*this, &CFNotification::onClicked));

    m_icon.setFromIconName(icon_name, CFICON_SIZE_DND);
    m_icon.setMarginLeft(10);
    m_icon.setMarginTop(10);
    m_icon.setValign(CFALIGN_START);
    m_actor.packStart(m_icon, false, false, 0);

    m_contentBox.setSizeRequest(300);

    m_time.setLabel(m_createTime.format(kDateFormat));
    m_time.setMarginLeft(5);
    m_time.setMarginTop(3);
    m_time.setHalign(CFALIGN_START);
    m_contentBox.packStart(m_time, false, false, 0);

    m_title.setLabel(title);
    m_title.setMarginLeft(5);
    m_title.setMarginTop(3);
    m_title.setHalign(CFALIGN_START);
    m_title.setLineWrap(true);

    m_title.setLines(3);  // reference ubuntu
    m_title.setEllipsize(Pango::ELLIPSIZE_END);
    m_contentBox.packStart(m_title, false, false, 0);

    m_content.setLabel(content);
    m_content.setMarginLeft(5);
    m_content.setMarginTop(3);
    m_content.setMarginBottom(5);
    m_content.setHalign(CFALIGN_START);
    m_content.setLineWrap(true);
    m_content.setJustify(CFJUSTIFY_LEFT);

    m_content.setLines(10);  // reference ubuntu
    m_content.setEllipsize(Pango::ELLIPSIZE_END);

    m_contentBox.packStart(m_content, false, false, 0);

    m_actor.packStart(m_contentBox, false, false, 0);

    add(m_actor);
}

CFNotification::~CFNotification() {
}

void CFNotification::setIcon(const Glib::ustring& label,
                             CFBuiltinIconSize icon_size) {
    m_icon.setFromIconName(label, icon_size);
}

CFImage& CFNotification::getIcon() {
    return m_icon;
}

Glib::ustring CFNotification::getTitle() {
    return m_title.getText();
}
void CFNotification::setTitle(const Glib::ustring& title) {
    return m_title.setText(title);
}

Glib::DateTime CFNotification::getCreateDateTime() {
    return m_createTime;
}

void CFNotification::setTime(const Glib::ustring& time) {
    m_time.setLabel(time);
}

void CFNotification::setReason(CFNotificationDestroyedReason reason) {
    m_reason = reason;
}

void CFNotification::onShow() {
    CFButton::onShow();
    setShowTime(m_isNeedShowTime);
}

void CFNotification::setShowTime(bool show_time) {
    m_isNeedShowTime = show_time;
    if (show_time) {
        m_time.show();
    } else {
        m_time.hide();
    }
}

void CFNotification::setTimeout(CFUInt ms_time) {
    if (ms_time <= 0) {
        return;
    }
    Glib::signal_timeout().connect(sigc::mem_fun(*this,
                                                 &CFNotification::onTimeout),
                                   ms_time);
}

CFNotificationUrgency CFNotification::getUrgency() {
    return m_urgency;
}

void CFNotification::setUrgency(CFNotificationUrgency urgency) {
    m_urgency = urgency;
}

// signal and handler
bool CFNotification::onTimeout() {
    m_reason = kTimeout;
    signalReason().emit(m_reason);
    return false;  // return false 仅执行1次
}

void CFNotification::onClicked() {
    m_reason = kClicked;
    signalReason().emit(m_reason);
}

sigc::signal<void, CFNotificationDestroyedReason>
CFNotification::signalReason() {
    return m_reasonSignal;
}

CF_NAMESPACE_END
