// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFPlacesSidebar that displays frequently-used places in the file system.
#include "Gui/CFPlacesSidebar.h"
#include <gtkmm.h>
#include <glibmm.h>
#include <PrivCommon/CFConvertor.h>

CF_NAMESPACE_BEGIN
// CFPlacesSidebarPriv
class CFPlacesSidebarPriv {
public:
    explicit CFPlacesSidebarPriv(CFPlacesSidebar* ps)
        : m_placesSidebar(ps) {
    }

    void onOpenLocation(const Glib::RefPtr<Gio::File>& location,
                        Gtk::PlacesOpenFlags open_flags);

    void onShowOtherLocationsWithFlags(Gtk::PlacesOpenFlags flags);

    void onPopulatePopup(Gtk::Menu* menu,
                         const Glib::RefPtr<Gio::File>& selected_item,
                         const Glib::RefPtr<Gio::Volume>& selected_volume);

private:
    CFPlacesSidebar* m_placesSidebar;
};

// CFPlacesSider
CFPlacesSidebar::CFPlacesSidebar() {
    setRealObject(new Gtk::PlacesSidebar());
}

CFPlacesSidebar::~CFPlacesSidebar() {
    setRealObject(NULL);
}

void CFPlacesSidebar::setOpenFlags(CFPlacesOpenFlags flags) {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        Gtk::PlacesOpenFlags gflags = toGtkPlacesOpenFlags(flags);
        obj->set_open_flags(gflags);
    }
    return;
}

CFPlacesOpenFlags CFPlacesSidebar::getOpenFlags() {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        Gtk::PlacesOpenFlags gflags = obj->get_open_flags();
        CFPlacesOpenFlags flags = toCdosPlacesOpenFlags(gflags);
        return flags;
    }
    return CFPLACES_OPEN_NORMAL;
}

void CFPlacesSidebar::setLocation(Glib::RefPtr<Gio::File>& file) {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        obj->set_location(file);
    }
    return;
}

Glib::RefPtr<Gio::File> CFPlacesSidebar::getLocation() {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    Glib::RefPtr<Gio::File> location = {};
    if (NULL != obj) {
        location = obj->get_location();
    }
    return location;
}

void CFPlacesSidebar::setShowDesktop(bool show_desktop) {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        obj->set_show_desktop(show_desktop);
    }
    return;
}

bool CFPlacesSidebar::getShowDesktop() {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        bool res = obj->get_show_desktop();
        return res;
    }
    return true;
}

void CFPlacesSidebar::addShortcut(const Glib::RefPtr<Gio::File>& location) {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        obj->add_shortcut(location);
    }
    return;
}

void CFPlacesSidebar::removeShortcut(const Glib::RefPtr<Gio::File>& location) {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        obj->remove_shortcut(location);
    }
    return;
}

std::vector<Glib::RefPtr<Gio::File>> CFPlacesSidebar::listShortcuts() {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    std::vector<Glib::RefPtr<Gio::File>> list = {};
    if (NULL != obj) {
        list = obj->list_shortcuts();
    }

    return list;
}

void CFPlacesSidebar::setShowConnectToServer(bool show_connect_to_server) {
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        obj->set_show_connect_to_server(show_connect_to_server);
    }
    return;
}

bool CFPlacesSidebar::getShowConnectToServer() {
    bool res = true;
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        res = obj->get_show_connect_to_server();
        return res;
    }
    return res;
}

// Init and finalize
void CFPlacesSidebar::onRealObjectInit() {
    CFScrolledView::onRealObjectInit();
    Gtk::PlacesSidebar* obj = static_cast<Gtk::PlacesSidebar*>(getObj());
    if (NULL != obj) {
        obj->signal_drag_action_requested().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onDragActionRequestd));

        obj->signal_drag_perform_drop().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onDragPerformDrop));

        obj->signal_drag_action_ask().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onDragActionAsk));

        obj->signal_show_connect_to_server().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onShowConnectToServer));

        obj->signal_show_error_message().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onShowErrorMessage));

        obj->signal_show_enter_location().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onShowEnterLocation));

        obj->signal_show_other_locations().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onShowOtherLocations));

        obj->signal_mount().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onMount));

        obj->signal_unmount().connect(
            sigc::mem_fun(*this, &CFPlacesSidebar::onUnmount));
        obj->signal_open_location().connect(
            sigc::mem_fun(*m_priv, &CFPlacesSidebarPriv::onOpenLocation));

        obj->signal_show_other_locations_with_flags().connect(
            sigc::mem_fun(*m_priv,
                          &CFPlacesSidebarPriv::onShowOtherLocationsWithFlags));
    }
}

// CFPlacesSidebarPriv signal
void CFPlacesSidebar::onRealObjectFinalize() {
}

void CFPlacesSidebarPriv::onOpenLocation(
    const Glib::RefPtr<Gio::File>& location, Gtk::PlacesOpenFlags open_flags) {
    m_placesSidebar->signalOpenLocation().emit(location,
                                               toCdosPlacesOpenFlags(
                                                   open_flags));
}

void CFPlacesSidebarPriv::onShowOtherLocationsWithFlags(
    Gtk::PlacesOpenFlags flags) {
    m_placesSidebar->signalShowOtherLocationsWithFlags().emit(
        toCdosPlacesOpenFlags(flags));
}

void CFPlacesSidebarPriv::onPopulatePopup(
    Gtk::Menu* menu,
    const Glib::RefPtr<Gio::File>& selected_item,
    const Glib::RefPtr<Gio::Volume>& selected_volume) {
    CFMenu* cmenu = static_cast<CFMenu*>(CFObject::getCFObject(menu));
    m_placesSidebar->signalPolulatePopup().emit(cmenu,
                                                selected_item,
                                                selected_volume);
}

// CFPlacesSider signal
sigc::signal<int,
             const Glib::RefPtr<Gdk::DragContext>&,
             const Glib::RefPtr<Gio::File>&,
             const std::vector<Glib::RefPtr<Gio::File>>&>
CFPlacesSidebar::signalDragActionRequested() {
    return m_dragActionRequestedSignal;
}

sigc::signal<void,
             const Glib::RefPtr<Gio::File>&,
             const std::vector<Glib::RefPtr<Gio::File>>&,
             int>
CFPlacesSidebar::signalDragPerformDrop() {
    return m_dragPerformDropSignal;
}

sigc::signal<int, int> CFPlacesSidebar::signalDragActionAsk() {
    return m_dragActionAskSignal;
}

sigc::signal<void> CFPlacesSidebar::signalShowConnectToServer() {
    return m_showConnectToServerSignal;
}

sigc::signal<void, const Glib::ustring&, const Glib::ustring&>
CFPlacesSidebar::signalShowErrorMessage() {
    return m_showErrorMessageSignal;
}

sigc::signal<void> CFPlacesSidebar::signalShowEnterLocation() {
    return m_showEnterLocationSignal;
}

sigc::signal<void> CFPlacesSidebar::signalShowOtherLocations() {
    return m_showOtherLocationsSignal;
}

sigc::signal<void, const Glib::RefPtr<Gio::MountOperation>&>
CFPlacesSidebar::signalMount() {
    return m_mountSignal;
}

sigc::signal<void, const Glib::RefPtr<Gio::MountOperation>&>
CFPlacesSidebar::signalUnmount() {
    return m_unmountSignal;
}

int CFPlacesSidebar::onDragActionRequestd(
    const Glib::RefPtr<Gdk::DragContext>& context,
    const Glib::RefPtr<Gio::File>& dest_file,
    const std::vector<Glib::RefPtr<Gio::File>>& source_file_list) {
    int res = signalDragActionRequested().emit(context,
                                               dest_file,
                                               source_file_list);
    return res;
}

void CFPlacesSidebar::onDragPerformDrop(
    const Glib::RefPtr<Gio::File>& dest_file,
    const std::vector<Glib::RefPtr<Gio::File>>& source_file_list,
    int action) {
    signalDragPerformDrop().emit(dest_file, source_file_list, action);
}

int CFPlacesSidebar::onDragActionAsk(int actions) {
    signalDragActionAsk().emit(actions);
    return 0;  // Fixme
}

void CFPlacesSidebar::onShowConnectToServer() {
    signalShowConnectToServer();
}

void CFPlacesSidebar::onShowErrorMessage(const Glib::ustring& primary,
                                         const Glib::ustring& secondary) {
    signalShowErrorMessage().emit(primary, secondary);
}

void CFPlacesSidebar::onShowEnterLocation() {
    signalShowEnterLocation().emit();
}

void CFPlacesSidebar::onShowOtherLocations() {
    signalShowEnterLocation().emit();
}

void CFPlacesSidebar::onMount(
    const Glib::RefPtr<Gio::MountOperation>& mount_operation) {
    signalMount().emit(mount_operation);
}

void CFPlacesSidebar::onUnmount(
    const Glib::RefPtr<Gio::MountOperation>& mount_operation) {
    signalUnmount().emit(mount_operation);
}

sigc::signal<void, const Glib::RefPtr<Gio::File>&, CFPlacesOpenFlags>
CFPlacesSidebar::signalOpenLocation() {
    return m_openLocationSignal;
}

sigc::signal<void, CFPlacesOpenFlags>
CFPlacesSidebar::signalShowOtherLocationsWithFlags() {
    return m_showOtherLocationsWithFlagsSignal;
}

sigc::signal<void,
             CFMenu*,
             const Glib::RefPtr<Gio::File>&,
             const Glib::RefPtr<Gio::Volume>&>
CFPlacesSidebar::signalPolulatePopup() {
    return m_polulatePopupSignal;
}

CF_NAMESPACE_END
