// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFRange is a base class for widgets which visullize an adjustment.
#include <Gui/CFRange.h>
#include "gtkextends/CFGtkRange.h"
#include <exception>
#include <iostream>

CF_NAMESPACE_BEGIN

CFRange::CFRange() {
    setRealObject(new CFGtkRange());
}

CFRange::~CFRange() {
    setRealObject(NULL);
}

static bool occurException(Gtk::Widget* obj) {
    try {
        if (obj == NULL) {
            std::string error_msg = "occur error, argument error.";
            throw std::invalid_argument(error_msg);
        }
        return true;
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
        return false;
    }
}

void CFRange::setIncrements(double step, double page) {
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur)
        return;
    obj->set_increments(step, page);
}

void CFRange::setRange(double min, double max) {
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur)
        return;
    obj->set_range(min, max);
}

void CFRange::setValue(double value) {
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur)
        return;
    obj->set_value(value);
}

double CFRange::getValue() {
    double value = -1;
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur) {
        return value;
    }
    value = obj->get_value();
    return value;
}

void CFRange::setRoundDigits(int round_digits) {
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur)
        return;

    obj->set_round_digits(round_digits);
}

int CFRange::getRoundDigits() {
    int digit = -1;
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur)
        return -1;

    digit = obj->get_round_digits();
    return digit;
}

void CFRange::onRealObjectInit() {
    CFWidget::onRealObjectInit();  // 调用父类自带
    Gtk::Range* obj = static_cast<Gtk::Range*>(getObj());
    bool exception_occur = occurException(obj);
    if (!exception_occur)
        return;

    obj->signal_value_changed().connect(
        sigc::mem_fun(*this, &CFRange::onChangeValue));
}

void CFRange::onRealObjectFinalize() {
    CFWidget::onRealObjectFinalize();
}

sigc::signal<void> CFRange::signalValueChanged() {
    return m_valueChangedSignal;
}

void CFRange::onChangeValue() {
    signalValueChanged().emit();
}

CF_NAMESPACE_END
