// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFScale is base class for widgets which visualize an adjustment.
#include "Gui/CFScale.h"
#include <gtkmm.h>
#include <exception>
#include <iostream>
#include "PrivCommon/CFConvertor.h"

CF_NAMESPACE_BEGIN

CFScale::CFScale() {
    setRealObject(new Gtk::Scale());
}

CFScale::~CFScale() {
    setRealObject(NULL);
}

CFScale::CFScale(CFOrientation orientation) {
    Gtk::Orientation orientation_ = toGtkOrientation(orientation);
    setRealObject(new Gtk::Scale(orientation_));
}

void CFScale::setDigits(int digits) {
    try {
        Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
        if (obj == NULL) {
            std::string error_msg = "CFScale != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        obj->set_digits(digits);
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
    }
}

int CFScale::getDigits() {
    int digit = -1;
    try {
        Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
        if (obj == NULL) {
            std::string error_msg = "CFScale != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        digit = obj->get_digits();
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
    }
    return digit;
}

void CFScale::setDrawValue(bool draw_value) {
    try {
        Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
        if (obj == NULL) {
            std::string error_msg = "CFScale != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        obj->set_draw_value(draw_value);
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
    }
}

bool CFScale::getDrawValue() {
    try {
        Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
        if (obj == NULL) {
            std::string error_msg = "CFScale != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        return obj->get_draw_value();
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
        return true;
    }
}

void CFScale::addMark(double value,
                      CFPositionType position,
                      const Glib::ustring& markup) {
    try {
        Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
        if (obj == NULL) {
            std::string error_msg = "CFScale != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        Gtk::PositionType position_ = static_cast<Gtk::PositionType>(position);
        obj->add_mark(value, position_, markup);
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
    }
}

void CFScale::clearMark() {
    try {
        Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
        if (obj == NULL) {
            std::string error_msg = "CFScale != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        obj->clear_marks();
    } catch (const std::exception& e) {
        std::cout << e.what() << std::endl;
    }
}

void CFScale::setInverted(bool setting) {
    Gtk::Scale* obj = static_cast<Gtk::Scale*>(getObj());
    if (obj != NULL) {
        obj->set_inverted(setting);
    }
}

CF_NAMESPACE_END
