// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// Adds scrollbars to its child widget.
#include "Gui/CFScrolledView.h"
#include <gtkmm.h>
#include "PrivCommon/CFConvertor.h"
#include "Core/CFObject.h"

CF_NAMESPACE_BEGIN

// class CFScrolledViewPriv
class CFScrolledViewPriv {
public:
    explicit CFScrolledViewPriv(CFScrolledView* sv)
        : m_scrollView(sv) {
    }

    void onEdgeOvershot(Gtk::PositionType cposition);

    void onEdgeReached(Gtk::PositionType cposition);

private:
    CFScrolledView* m_scrollView;
};

void CFScrolledViewPriv::onEdgeOvershot(Gtk::PositionType cposition) {
    m_scrollView->signalEdgeOvershot().emit(toCdosPositionType(cposition));
}

void CFScrolledViewPriv::onEdgeReached(Gtk::PositionType cposition) {
    m_scrollView->signalEdgeReached().emit(toCdosPositionType(cposition));
}


// class CFScrolledView
CFScrolledView::CFScrolledView()
    : m_priv(new CFScrolledViewPriv(this)) {
    setRealObject(new Gtk::ScrolledWindow());
}

CFScrolledView::~CFScrolledView() {
    setRealObject(NULL);
}

// void CFScrolledView::setHadjustment(CFAdjustment& hadjustment) {
//     Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
//     while (NULL != obj) {
//
//     }
// }

// CFAdjustment CFScrolledView::getHajustment() {
//     Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
//     while (NULL != obj) {
//         Glib::RefPtr<Gtk::Adjustment> adj = obj->get_hadjustment();
//     }
// }

// void CFScrolledView::setVadjustment(CFAdjustment& vhadjustment) {
//     Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
//     while (NULL != obj) {
//     }
// }

// CFAdjustment CFScrolledView::getVajustment() {
//     Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
//     while (NULL != obj) {
//         Glib::RefPtr<Gtk::Adjustment> adj = obj->get_hadjustment();
//     }
// }

void CFScrolledView::setPolicy(CFPolicyType hscrollbar_policy,
                               CFPolicyType vscrollbar_policy) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        Gtk::PolicyType h_scrollbarPolicy = toGtkPolicyType(hscrollbar_policy);
        Gtk::PolicyType v_scrollbarPolicy = toGtkPolicyType(vscrollbar_policy);
        obj->set_policy(h_scrollbarPolicy, v_scrollbarPolicy);
    }
}

void CFScrolledView::getPolicy(CFPolicyType& hscrollbar_pollcy,
                               CFPolicyType& vscrollbar_pollcy) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        Gtk::PolicyType h_scrollbarPolicy, v_scrollbarPolicy;
        obj->get_policy(h_scrollbarPolicy, v_scrollbarPolicy);
        hscrollbar_pollcy = toCdosPolicyType(h_scrollbarPolicy);
        vscrollbar_pollcy = toCdosPolicyType(v_scrollbarPolicy);
    }
}

void CFScrolledView::setPlacement(CFCornerType window_placement) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        Gtk::CornerType placement = toGtkCornerType(window_placement);
        obj->set_placement(placement);
    }
}

void CFScrolledView::unsetPlacement() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->unset_placement();
    }
}

CFCornerType CFScrolledView::getPlacement() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    CFCornerType ctype = CFCORNER_TOP_LEFT;
    while (NULL != obj) {
        Gtk::CornerType type = obj->get_placement();
        ctype = toCdosCornerType(type);
    }
    return ctype;
}

void CFScrolledView::setShadowType(CFShadowType type) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    Gtk::ShadowType gtype = toGtkShadowType(type);
    while (NULL != obj) {
        obj->set_shadow_type(gtype);
    }
}

CFShadowType CFScrolledView::getShadowType() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    CFShadowType ctype = CFSHADOW_NONE;
    while (NULL != obj) {
        Gtk::ShadowType gtype = obj->get_shadow_type();
        ctype = toCdosShadowType(gtype);
    }
    return ctype;
}

// override Cdos::CFConter::add
void CFScrolledView::add(CFWidget& widget) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    Gtk::Widget* wobj = static_cast<Gtk::Widget*>(widget.getObj());
    while (NULL != obj) {
        obj->add(*wobj);
    }
}

void CFScrolledView::removeWithViewport() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->remove_with_viewport();
    }
}

CFScrollbar* CFScrolledView::getVscrollbar() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        Gtk::Scrollbar* bar = obj->get_vscrollbar();
        CFScrollbar* cbar
            = static_cast<CFScrollbar*>(CFObject::getCFObject(bar));
        return cbar;
    }
    return NULL;
}

CFScrollbar* CFScrolledView::setHscrollbar() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        Gtk::Scrollbar* bar = obj->get_hscrollbar();
        CFScrollbar* cbar
            = static_cast<CFScrollbar*>(CFObject::getCFObject(bar));
        return cbar;
    }
    return NULL;
}

int CFScrolledView::getMinContentWidth() {
    int width = -1;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        width = obj->get_min_content_width();
    }
    return width;
}

void CFScrolledView::setMinContentWidth(int width) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_min_content_width(width);
    }
}

int CFScrolledView::getMinContentHeight() {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        int height = obj->get_min_content_height();
        return height;
    }
    return -1;
}

void CFScrolledView::setMinContentHeight(int height) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_min_content_height(height);
    }
}

void CFScrolledView::setKineticScrolling(bool kinetic_scrolling) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_kinetic_scrolling(kinetic_scrolling);
    }
}

bool CFScrolledView::getKineticScrolling() {
    bool res = true;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        res = obj->get_kinetic_scrolling();
    }
    return res;
}

void CFScrolledView::setCaptureButtonPress(bool capture_button_press) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_capture_button_press(capture_button_press);
    }
}

bool CFScrolledView::getCaptureButtonPress() {
    bool res = false;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        res = obj->get_capture_button_press();
    }
    return res;
}

void CFScrolledView::setOverlayScrolling(bool overlay_scrolling) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_overlay_scrolling(overlay_scrolling);
    }
}

bool CFScrolledView::getOverlayScrolling() {
    bool res = true;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        res = obj->get_overlay_scrolling();
    }
    return res;
}

void CFScrolledView::setMaxContentWidth(int width) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_max_content_width(width);
    }
}

int CFScrolledView::getMaxContentWidth() {
    int width = -1;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        width = obj->get_max_content_width();
    }
    return width;
}

void CFScrolledView::setMaxContentHeight(int height) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_max_content_height(height);
    }
}

int CFScrolledView::getMaxContentHeight() {
    int height = -1;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        height = obj->get_max_content_height();
    }
    return height;
}

void CFScrolledView::setPropagateNaturalWidth(bool propagete) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_propagate_natural_width(propagete);
    }
}

bool CFScrolledView::getPropagateNaturalWidth() {
    bool res = true;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        res = obj->get_propagate_natural_width();
    }
    return res;
}

void CFScrolledView::setPropagateNaturalHeight(bool propagete) {
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->set_propagate_natural_height(propagete);
    }
}

bool CFScrolledView::getPropagateNaturalHeight() {
    bool res = false;
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        res = obj->get_propagate_natural_height();
    }
    return res;
}

sigc::signal<void, CFPositionType> CFScrolledView::signalEdgeOvershot() {
    return m_edgeOvershotSignal;
}

sigc::signal<void, CFPositionType> CFScrolledView::signalEdgeReached() {
    return m_edgeReachedSignal;
}

void CFScrolledView::onRealObjectInit() {
    // CFBin::onRealObjectInit(); this function cannot contain override
    Gtk::ScrolledWindow* obj = static_cast<Gtk::ScrolledWindow*>(getObj());
    while (NULL != obj) {
        obj->signal_edge_overshot().connect(
            sigc::mem_fun(*m_priv, &CFScrolledViewPriv::onEdgeOvershot));
        obj->signal_edge_reached().connect(
            sigc::mem_fun(*m_priv, &CFScrolledViewPriv::onEdgeReached));
    }
}

void CFScrolledView::onRealObjectFinalize() {
}

CF_NAMESPACE_END
