// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// A "light switch"-style toggle.
#include "Gui/CFSwitch.h"
#include <gtkmm.h>

CF_NAMESPACE_BEGIN

CFSwitch::CFSwitch() {
    setRealObject(new Gtk::Switch());
}

CFSwitch::~CFSwitch() {
    setRealObject(NULL);
}

void CFSwitch::setActive(bool is_active) {
    Gtk::Switch* obj = static_cast<Gtk::Switch*>(getObj());
    if (NULL != obj) {
        obj->set_active(is_active);
    }
    return;
}

bool CFSwitch::getActive() {
    Gtk::Switch* obj = static_cast<Gtk::Switch*>(getObj());
    bool res = true;
    if (NULL != obj) {
        res = obj->get_active();
    }

    return res;
}

void CFSwitch::setState(bool state) {
    Gtk::Switch* obj = static_cast<Gtk::Switch*>(getObj());
    if (NULL != obj) {
        obj->set_state(state);
    }
    return;
}

bool CFSwitch::getState() {
    Gtk::Switch* obj = static_cast<Gtk::Switch*>(getObj());
    bool state = true;
    if (NULL != obj) {
        state = obj->get_state();
    }
    return state;
}

// signal and handler
sigc::signal<bool, bool> CFSwitch::signalStateSet() {
    return m_stateSetSignal;
}

bool CFSwitch::onStateSet(bool state) {
    bool res = signalStateSet().emit(state);
    return res;
}

void CFSwitch::onRealObjectInit() {
    CFWidget::onRealObjectInit();
    Gtk::Switch* obj = static_cast<Gtk::Switch*>(getObj());
    if (NULL != obj) {
        obj->signal_state_set().connect(
            sigc::mem_fun(*this, &CFSwitch::onStateSet));
    }
}

void CFSwitch::onRealObjectFinalize() {
}

CF_NAMESPACE_END
