// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// A CFToggleButton will remain 'pressed-in' when clicked.
#include "Gui/CFToggleButton.h"
#include <exception>
#include <iostream>
#include <gtkmm.h>

CF_NAMESPACE_BEGIN

CFToggleButton::CFToggleButton()
    : CFButton() {
    setRealObject(new Gtk::ToggleButton());
}

CFToggleButton::CFToggleButton(const Glib::ustring& label, bool mnemonic)
    : CFButton(label, mnemonic) {
    setRealObject(new Gtk::ToggleButton(label, mnemonic));
}

CFToggleButton::~CFToggleButton() {
    setRealObject(NULL);
}

void CFToggleButton::setMode(bool draw_indicator) {
    Gtk::ToggleButton* obj = static_cast<Gtk::ToggleButton*>(getObj());
    if (NULL != obj) {
        obj->set_mode(draw_indicator);
    }
    return;
}

bool CFToggleButton::getMode() {
    Gtk::ToggleButton* obj = static_cast<Gtk::ToggleButton*>(getObj());
    if (NULL != obj) {
        return obj->get_mode();
    }
    return true;
}

void CFToggleButton::setActive(bool is_active) {
    Gtk::ToggleButton* obj = static_cast<Gtk::ToggleButton*>(getObj());
    if (NULL != obj) {
        obj->set_active(is_active);
    }
}

bool CFToggleButton::getActive() {
    Gtk::ToggleButton* obj = static_cast<Gtk::ToggleButton*>(getObj());
    if (NULL != obj) {
        return obj->get_active();
    }
    return true;
}

void CFToggleButton::onRealObjectInit() {
    CFButton::onRealObjectInit();
    Gtk::ToggleButton* obj = static_cast<Gtk::ToggleButton*>(getObj());
    if (NULL != obj) {
        obj->signal_toggled().connect(
            sigc::mem_fun(*this, &CFToggleButton::toggled));
    }
}

void CFToggleButton::onRealObjectFinalize() {
    CFButton::onRealObjectFinalize();
}

sigc::signal<void> CFToggleButton::signalToggled() {
    return m_toggleButtonSignal;
}

void CFToggleButton::toggled() {
    signalToggled().emit();
}

CF_NAMESPACE_END
