// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFVolumeItem is a slider class.

#include "Gui/CFVolumeButtonItem.h"
#include <iostream>
#include <iomanip>
#include "Gui/CFEnums.h"

CF_NAMESPACE_BEGIN

CFVolumeButtonItem::CFVolumeButtonItem(CFOrientation orientation,
                                       CFVolumeButtonItemType type)
    : CFBox(CFORIENTATION_VERTICAL, 0)
    , m_container(orientation, 0)
    , m_slider(orientation) {
    m_type = type;
    if (orientation != CFORIENTATION_VERTICAL) {
        // set size
        m_title_text_width = 300;
        m_title_icons_size = 30;

        // set title
        m_title_val.setLabel("Title");
        m_title_val.setJustify(CFJUSTIFY_RIGHT);
        m_title_val.setSizeRequest(m_title_text_width, m_title_icons_size);
        m_title.put(m_title_val, 0, 0);
        m_title_icons.setFromIconName("audio-volume-muted", CFICON_SIZE_BUTTON);
        m_title_icons.setSizeRequest(m_title_icons_size * 3,
                                     m_title_icons_size);
        // if parameter input 5 / 8 * width is wrong
        m_title.put(m_title_icons, 5 * m_title_text_width / 8, 0);
        m_container.add(m_title);

        // set scale
        m_slider.setRange(0, 1);
        m_slider.setIncrements(0.05, 0.05);
        m_slider.setDrawValue(false);  // set weather display current value.
        m_slider.signalValueChanged().connect(
            sigc::mem_fun(*this, &CFVolumeButtonItem::onScaleChanged));
        // Fixme
        m_slider.setSizeRequest(m_title_text_width + m_title_icons_size - 50);
        m_slider_fixed.put(m_slider, 10, 0);
        m_container.add(m_slider_fixed);

        // checkbox
        m_mute.setLabel("Item Mutate");
        m_mute.setSizeRequest(-1, 25);
        m_mute.signalToggled().connect(
            sigc::mem_fun(*this, &CFVolumeButtonItem::onMuted));
        m_mute_fixed.put(m_mute, 15, 0);
        m_container.add(m_mute_fixed);
    } else {
        // set size
        m_title_text_width = 110;
        m_title_icons_size = 26;

        // set title
        m_title_val.setLabel("Title");
        m_title_val.setJustify(CFJUSTIFY_LEFT);
        m_title_val.setSizeRequest(m_title_text_width, m_title_icons_size);
        m_title.put(m_title_val, 0, 0);
        m_title_icons.setFromIconName("audio-volume-muted", CFICON_SIZE_BUTTON);
        m_title_icons.setSizeRequest(m_title_icons_size * 3,
                                     m_title_icons_size);
        // if parameter input 5 / 8 * width is wrong
        m_title.put(m_title_icons, 5 * m_title_text_width / 8, 0);
        m_container.add(m_title);

        // set scale
        m_slider.setRange(0, 1);
        m_slider.setIncrements(0.05, 0.05);
        m_slider.setDrawValue(false);  // set weather display current value.
        // Fixme
        m_slider.setInverted(true);  // change direction 180°
        m_slider.setSizeRequest(15, 174);
        m_slider.signalValueChanged().connect(
            sigc::mem_fun(*this, &CFVolumeButtonItem::onScaleChanged));
        m_slider_fixed.put(m_slider, 55, 0);
        m_container.add(m_slider_fixed);

        // checkbox
        m_mute.setLabel("Item Mutate");
        m_mute.setSizeRequest(-1, 25);
        m_mute.signalToggled().connect(
            sigc::mem_fun(*this, &CFVolumeButtonItem::onMuted));
        m_mute_fixed.put(m_mute, 15, 0);
        m_container.add(m_mute_fixed);
    }

    add(m_container);
    showAll();
}

CFVolumeButtonItem::~CFVolumeButtonItem() {
}

void CFVolumeButtonItem::onScaleChanged() {
    double get_value = m_slider.getValue();
    // std::cout << get_value << std::endl;
    CFInt32 current_val = static_cast<CFInt32>(floor(get_value * 100));
    // std::cout << current_val << std::endl;
    Glib::ustring text;
    if (!m_type || m_type == kVolumeItem) {
        text = Glib::ustring::compose("声音: %1%%",
                                      Glib::ustring::format(std::setfill(L'0'),
                                                            std::setw(3),
                                                            current_val));
    } else {
        text = Glib::ustring::compose("麦克风: %1%%",
                                      Glib::ustring::format(std::setfill(L'0'),
                                                            std::setw(3),
                                                            current_val));
    }

    m_title_val.setLabel(text);
    if (get_value == 0) {
        m_mute.setActive(true);
        setTitleIcons(0);
    } else {
        m_mute.setActive(false);
        setTitleIcons(current_val);
    }
}

void CFVolumeButtonItem::onMuted() {
    bool is_active = m_mute.getActive();
    // std::cout << is_active << std::endl;
    if (is_active) {
        setSliderValue(0);
    }
}

// getter and setter
double CFVolumeButtonItem::getSliderValue() {
    double get_value = m_slider.getValue();
    return get_value;
}

void CFVolumeButtonItem::setSliderValue(double value) {
    m_slider.setValue(value);
}

// type set in initialized
CFVolumeButtonItemType CFVolumeButtonItem::getItemType() {
    return this->m_type;
}

// no get title icons
void CFVolumeButtonItem::setTitleIcons(int current_val) {
    if (current_val == 0) {
        m_title_icons.setFromIconName("audio-volume-muted", CFICON_SIZE_BUTTON);
        return;
    }

    if (current_val > 100) {
        current_val = 100;
    }

    CFInt32 icon_status = static_cast<CFInt32>(floor(current_val / 33) + 1);
    switch (icon_status) {
    case 1:
        m_title_icons.setFromIconName("audio-volume-low", CFICON_SIZE_BUTTON);
        break;
    case 2:
        m_title_icons.setFromIconName("audio-volume-medium",
                                      CFICON_SIZE_BUTTON);
        break;
    default:
        m_title_icons.setFromIconName("audio-volume-high", CFICON_SIZE_BUTTON);
    }
    return;
}

bool CFVolumeButtonItem::getMuteStatus() {
    bool is_muted = m_mute.getActive();
    return is_muted;
}

void CFVolumeButtonItem::setMuteStatus(bool is_muted) {
    m_mute.setActive(is_muted);
}

void CFVolumeButtonItem::showMicrophone(bool is_show) {
    if (m_type == kVolumeItem) {
        return;
    }
    is_show ? m_container.show() : m_container.hide();
    return;
}

void CFVolumeButtonItem::setMuteLabel(std::string label) {
    m_mute.setLabel(label);
}

CF_NAMESPACE_END
