#include "Gui/CFWidget.h"
#include <gtkmm.h>
#include "gtkextends/CFGtkWidget.h"
#include "CFConvertor.h"
#include <iostream>

CF_NAMESPACE_BEGIN

CFWidget::CFWidget()
    : CFObject() {
    setRealObject(new CFGtkWidget());
}

CFWidget::~CFWidget() {
    setRealObject(NULL);
}

void CFWidget::setEvents(Gdk::EventMask events) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_events(events);
    }
}

void CFWidget::addEvents(Gdk::EventMask events) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->add_events(events);
    }
}

int CFWidget::getAllocatedWidth() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_allocated_width();
    }
    return -1;
}

int CFWidget::getAllocatedHeight() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_allocated_height();
    }
    return -1;
}

void CFWidget::setSizeRequest(int width, int height) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_size_request(width, height);
    }
}

void CFWidget::queueDraw() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->queue_draw();
    }
}

void CFWidget::queueDrawArea(int x, int y, int width, int height) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->queue_draw_area(x, y, width, height);
    }
}

void CFWidget::reparent(CFWidget& new_parent) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    Gtk::Widget* newPObj = static_cast<Gtk::Widget*>(new_parent.getObj());
    if (NULL != obj && NULL != newPObj) {
        obj->reparent(*newPObj);
    }
}

void CFWidget::setCanFocus(bool can_focus) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_can_focus(can_focus);
    }
}

bool CFWidget::getCanFocus() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_can_focus();
    }
    return false;
}

bool CFWidget::hasFocus() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->has_focus();
    }
    return false;
}

bool CFWidget::isFocus() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->is_focus();
    }
    return false;
}

void CFWidget::grabFocus() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->grab_focus();
    }
}

bool CFWidget::isContainer() {
    return false;
}

void CFWidget::setVisible(bool visible) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_visible(visible);
    }
}

bool CFWidget::getVisible() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_visible();
    }
    return false;
}

bool CFWidget::isVisible() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->is_visible();
    }
    return false;
}

void CFWidget::show() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->show();
    }
}

void CFWidget::showAll() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->show_all();
    }
}

void CFWidget::hide() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->hide();
    }
}

void CFWidget::setName(const Glib::ustring& name) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_name(name);
    }
}

void CFWidget::setHalign(CFAlign align) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_halign(toGtkAlign(align));
    }
}

CFAlign CFWidget::getHalign() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return toCdosAlign(obj->get_halign());
    }
    return CFALIGN_START;
}

void CFWidget::setValign(CFAlign align) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_valign(toGtkAlign(align));
    }
}

CFAlign CFWidget::getValign() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return toCdosAlign(obj->get_valign());
    }
    return CFALIGN_START;
}

void CFWidget::setTooltipText(const Glib::ustring& text) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_tooltip_text(text);
    }
}

Glib::ustring CFWidget::getTooltipText() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_tooltip_text();
    }
    return "";
}

void CFWidget::setTooltipMarkup(const Glib::ustring& markup) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_tooltip_markup(markup);
    }
}

Glib::ustring CFWidget::getTooltipMarkup() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_tooltip_markup();
    }
    return "";
}

void CFWidget::setHasTooltip(bool has_tooltip) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_has_tooltip(has_tooltip);
    }
}
bool CFWidget::getHasTooltip() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_has_tooltip();
    }
    return false;
}

void CFWidget::setSensitive(bool sensitive) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_sensitive(sensitive);
    }
}

bool CFWidget::getSensitive() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_sensitive();
    }
    return false;
}

bool CFWidget::isSensitive() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->is_sensitive();
    }
    return false;
}

void CFWidget::setMarginLeft(int margin) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_margin_left(margin);
    }
}

int CFWidget::getMarginLeft() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_margin_left();
    }
    return 0;
}

void CFWidget::setMarginRight(int margin) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_margin_right(margin);
    }
}

int CFWidget::getMarginRight() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_margin_right();
    }
    return 0;
}

void CFWidget::setMarginTop(int margin) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_margin_top(margin);
    }
}

int CFWidget::getMarginTop() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_margin_top();
    }
    return 0;
}

void CFWidget::setMarginBottom(int margin) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->set_margin_bottom(margin);
    }
}

int CFWidget::getMarginBottom() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_margin_bottom();
    }
    return 0;
}

void CFWidget::addCssStylePath(const Glib::ustring& path, CFUInt priority) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        Glib::RefPtr<Gtk::CssProvider> provider = Gtk::CssProvider::create();
        Glib::RefPtr<Gtk::StyleContext> context = obj->get_style_context();
        provider->load_from_path(path);
        context->add_provider(provider, priority);
    }
}

void CFWidget::addCssStyleData(const Glib::ustring& data, CFUInt priority) {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        Glib::RefPtr<Gtk::CssProvider> provider = Gtk::CssProvider::create();
        Glib::RefPtr<Gtk::StyleContext> context = obj->get_style_context();
        provider->load_from_data(data);
        context->add_provider(provider, priority);
    }
}

void CFWidget::addCssStyleName(const Glib::ustring& name) {
    try {
        Gtk::Widget* widget = static_cast<Gtk::Widget*>(getObj());
        Glib::RefPtr<Gtk::StyleContext> style_context
            = widget->get_style_context();
        style_context->add_class(name);
        return;
    } catch (const std::logic_error& e) {
        std::cout << "class name cannot nullptr" << std::endl;
        return;
    } catch (const std::exception& e) {
        std::cout << "throw exception: " << e.what() << std::endl;
        return;
    }
}

void CFWidget::addCssStyleFromPath(const Glib::ustring& css_file_path,
                                   const Glib::ustring& name,
                                   CFUInt priority) {
    std::string error_msg;
    try {
        Gtk::Widget* widget = static_cast<Gtk::Widget*>(getObj());
        if (nullptr == widget) {
            error_msg = "widget != NULL is failed!";
            throw(error_msg);
        }

        Glib::RefPtr<Gtk::CssProvider> provider = Gtk::CssProvider::create();
        Glib::RefPtr<Gtk::StyleContext> style_context
            = widget->get_style_context();
        provider->load_from_path(css_file_path);
        style_context->add_provider(provider, priority);
        style_context->add_class(name);
        return;
    } catch (const Gtk::CssProviderError& e) {
        error_msg = "Please confirm css file path or class name is correct!";
        std::cout << error_msg << std::endl;
        return;
    } catch (const std::string& e) {
        std::cout << e << std::endl;
        return;
    } catch (const std::logic_error& e) {
        std::cout << "css_file_path is not nullptr" << std::endl;
        return;
    }
}

void CFWidget::listCssStyle() {
    try {
        Gtk::Widget* widget = static_cast<Gtk::Widget*>(getObj());
        if (widget == NULL) {
            std::string error_msg = "widget != NULL is failed.";
            throw std::invalid_argument(error_msg);
        }
        Glib::RefPtr<Gtk::StyleContext> style_context
            = widget->get_style_context();
        const std::vector<Glib::ustring> context_list
            = style_context->list_classes();
        for (auto& i : context_list) {
            std::cout << "Css style: " << i << std::endl;
        }
        return;
    } catch (const std::invalid_argument& e) {
        std::cout << e.what() << std::endl;
        return;
    } catch (const std::exception& e) {
        std::cout << "Internal Error." << std::endl;
        return;
    }
}

std::vector<Glib::ustring> CFWidget::getCssStyleVector() {
    Gtk::Widget* widget = static_cast<Gtk::Widget*>(getObj());
    if (widget != NULL) {
        Glib::RefPtr<Gtk::StyleContext> style_context
            = widget->get_style_context();
        std::vector<Glib::ustring> context_list = style_context->list_classes();
        return context_list;
    }
}

void CFWidget::removeCssStyle(const Glib::ustring& name) {
    Gtk::Widget* widget = static_cast<Gtk::Widget*>(getObj());
    if (widget != NULL) {
        Glib::RefPtr<Gtk::StyleContext> style_context
            = widget->get_style_context();
        style_context->remove_class(name);
    }
    return;
}

Glib::RefPtr<Gdk::Window> CFWidget::getWindow() {
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        return obj->get_window();
    }
    return Glib::RefPtr<Gdk::Window>(NULL);
}

void CFWidget::onRealObjectInit() {
    CFObject::onRealObjectInit();
    Gtk::Widget* obj = static_cast<Gtk::Widget*>(getObj());
    if (NULL != obj) {
        obj->signal_draw().connect(sigc::mem_fun(*this, &CFWidget::onDraw));
        obj->signal_event().connect(sigc::mem_fun(*this, &CFWidget::onEvent));
        obj->signal_button_press_event().connect(
            sigc::mem_fun(*this, &CFWidget::onButtonPressEvent));
        obj->signal_button_release_event().connect(
            sigc::mem_fun(*this, &CFWidget::onButtonReleaseEvent));
        obj->signal_delete_event().connect(
            sigc::mem_fun(*this, &CFWidget::onDeleteEvent));

        obj->signal_key_press_event().connect(
            sigc::mem_fun(*this, &CFWidget::onKeyPressEvent));
        obj->signal_key_release_event().connect(
            sigc::mem_fun(*this, &CFWidget::onKeyReleaseEvent));
        obj->signal_show().connect(sigc::mem_fun(*this, &CFWidget::onShow));
    }
}

void CFWidget::onRealObjectFinalize() {
    CFObject::onRealObjectFinalize();
}

sigc::signal<bool, const ::Cairo::RefPtr<::Cairo::Context>&>
CFWidget::signalDraw() {
    return m_drawSignal;
}

sigc::signal<bool, GdkEvent*> CFWidget::signalEvent() {
    return m_eventSignal;
}

sigc::signal<bool, GdkEventButton*> CFWidget::signalButtonPressEvent() {
    return m_buttonPressEventSignal;
}

sigc::signal<bool, GdkEventButton*> CFWidget::signalButtonReleaseEvent() {
    return m_buttonReleaseEventSignal;
}

sigc::signal<bool, GdkEventAny*> CFWidget::signalDeleteEvent() {
    return m_deleteEventSignal;
}

sigc::signal<bool, GdkEventKey*> CFWidget::signalKeyPressEvent() {
    return m_keyPressEventSignal;
}

sigc::signal<bool, GdkEventKey*> CFWidget::signalKeyReleaseEvent() {
    return m_keyReleaseEventSignal;
}

sigc::signal<void> CFWidget::signalShow() {
    return m_showSignal;
}

bool CFWidget::onDraw(const ::Cairo::RefPtr<::Cairo::Context>& cr) {
    return signalDraw().emit(cr);
}

bool CFWidget::onEvent(GdkEvent* event) {
    return signalEvent().emit(event);
}

bool CFWidget::onButtonPressEvent(GdkEventButton* event) {
    return signalButtonPressEvent().emit(event);
}

bool CFWidget::onButtonReleaseEvent(GdkEventButton* event) {
    return signalButtonReleaseEvent().emit(event);
}

bool CFWidget::onDeleteEvent(GdkEventAny* event) {
    return signalDeleteEvent().emit(event);
}

bool CFWidget::onKeyPressEvent(GdkEventKey* event) {
    return signalKeyPressEvent().emit(event);
}

bool CFWidget::onKeyReleaseEvent(GdkEventKey* event) {
    return signalKeyReleaseEvent().emit(event);
}

void CFWidget::onShow() {
    return signalShow().emit();
}

CF_NAMESPACE_END
