#include <Support/CFDate.h>
#include <time.h>

#define CFDATE_MONTH_MAX (12)
#define CFDATE_YEAR_MIN (1900)
#define CFDATE_YEAR_MAX (2050)
#define CFDATE_DELTA_VAL (12)
#define CFDATE_BASE_CENTRY (18)

static const CFUInt month_length[2][13] = {
    {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
};

static const CFUInt days_in_months[2][14] = {
    {0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365},
    {0, 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366}
};

CFDate::CFDate(CFInt y, CFInt m, CFInt d) {
    if (isValidDay(y, m, d)) {
        m_year = y;
        m_month = m;
        m_dayOfMonth = d;
        m_dayOfWeek = calcWeekDay(y, m, d);

    } else {
        time_t localtmp = time(NULL);
        struct tm* localtm = localtime(&localtmp);
        m_year = localtm->tm_year + CFDATE_YEAR_MIN;
        m_month = localtm->tm_mon + 1;
        m_dayOfMonth = localtm->tm_mday;
        m_dayOfWeek = (CFWeek)localtm->tm_wday;
    }
}

CFDate::CFDate() {
    time_t localtmp = time(NULL);
    struct tm* localtm = localtime(&localtmp);
    m_year = localtm->tm_year + CFDATE_YEAR_MIN;
    m_month = localtm->tm_mon + 1;
    m_dayOfMonth = localtm->tm_mday;
    m_dayOfWeek = (CFWeek)localtm->tm_wday;
}

void CFDate::setYear(CFInt y) {
    if (!isValidYear(y)) {
        return;
    }
    m_year = y;
    reCalcWeekDay();
}

void CFDate::setMonth(CFInt m) {
    if (!isValidMonth(m)) {
        return;
    }
    m_month = m;
    reCalcWeekDay();
}

void CFDate::setDayOfMonth(CFInt d) {
    if (!isValidDay(m_year, m_month, d)) {
        return;
    }
    m_dayOfMonth = d;
    reCalcWeekDay();
}

CFInt CFDate::getYear() {
    return m_year;
}

CFInt CFDate::getMonth() {
    return m_month;
}

CFInt CFDate::getDayOfMonth() {
    return m_dayOfMonth;
}

CFWeek CFDate::getWeekDay() {
    return m_dayOfWeek;
}

CFInt CFDate::getDayOfYear() {
    CFInt leapIndex = (isLeap(m_year) ? 1 : 0);
    return (days_in_months[leapIndex][m_month] + m_dayOfMonth);
}

bool CFDate::isValidYear(CFInt y) {
    return (CFDATE_YEAR_MIN <= y && CFDATE_YEAR_MAX > y);
}

bool CFDate::isValidMonth(CFInt m) {
    return (0 < m && CFDATE_MONTH_MAX >= m);
}

bool CFDate::isValidDay(CFInt y, CFInt m, CFInt d) {
    if (isValidYear(y) && isValidMonth(m)) {
        CFInt leapIndex = (isLeap(y) ? 1 : 0);
        CFInt daylenth = month_length[leapIndex][m];
        return (0 < d && daylenth >= d);
    }
    return false;
}

bool CFDate::isLeap(CFInt y) {
    return ((((y % 4) == 0) && ((y % 100) != 0)) || ((y % 400) == 0));
}

void CFDate::reCalcWeekDay() {
    m_dayOfWeek = calcWeekDay(m_year, m_month, m_dayOfMonth);
}

CFInt CFDate::countYearByModBet(CFInt nYearS, CFInt nYearE, CFInt nBet) {
    CFInt nCount = 0;
    nYearE = nYearE - 1;  // include nYearS, but not include nYearE
    CFInt nYearDec = nYearE - nYearS;
    if (0 > nYearDec || 0 >= nBet) {
        return 0;
    }
  
    CFInt nModS = nYearS % nBet;
    CFInt nModE = nYearE % nBet;
  
    if (nModS == 0 || nModE < nModS) {
        nCount = nYearDec/nBet + 1;  
    } else {
        nCount = nYearDec/nBet;
    }
  
    return nCount;  
}  
  
CFInt CFDate::countLeapYear(CFInt nYearS, CFInt nYearE) {
    CFInt nCount = 0;
    nCount = countYearByModBet(nYearS, nYearE, 4)
           - countYearByModBet(nYearS, nYearE, 100)
           + countYearByModBet(nYearS, nYearE, 400);
    return nCount;
}

CFWeek CFDate::calcWeekDay(CFInt y, CFInt m, CFInt d) {
    CFInt leapIndex = (isLeap(y) ? 1 : 0);
    CFInt leapYearCount = countLeapYear(1900, y);
    CFInt noLeapYearCount = (y - 1900) - leapYearCount;
    CFInt allDayCount = noLeapYearCount * 365 + leapYearCount * 366 + days_in_months[leapIndex][m] + d;
    return (CFWeek)(allDayCount % 7);
}